/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.controller;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import nan.tools.EventLogger;

import nan.view.ComposeFrame;

/**
 * The menu for the compose frame.
 *
 * @author John David Ratliff
 * @version 1.5, 1/6/04
 */
public class ComposeMenu extends JMenuBar implements ActionListener {
    private ComposeFrame parent;
    private JMenu actionMenu = buildActionMenu();
    private JMenu attachmentMenu = buildAttachmentMenu();
    private JMenu attachments;
    private static final String ACTION_SEND_ACTION = "_action_send";
    private static final String ACTION_CANCEL_ACTION = "_action_cancel";
    private static final String ATTACHMENT_ADD_ACTION = "_attach_add";

    /**
     * Creates a new ComposeMenu object.
     *
     * @param parent The ComposeFrame this is the Menu for.
     */
    public ComposeMenu(ComposeFrame parent) {
        this.parent = parent;

        add(actionMenu);
        add(attachmentMenu);
    }

    /**
     * Adds an attachment to the remove menu.
     *
     * @param name The name of the attachment.
     */
    public void addAttachment(String name) {
        JMenuItem item = new JMenuItem(name);
        item.setActionCommand(name);
        item.addActionListener(this);

        attachments.add(item);
    }

    /**
     * Removes an attachment from the remove menu.
     *
     * @param item The JMenuItem the attachment is listed on.
     */
    private void removeAttachment(JMenuItem item) {
        attachments.remove(item);
    }

    /**
     * Builds the Action menu.
     *
     * @return The Action menu.
     */
    private JMenu buildActionMenu() {
        JMenu menu = new JMenu("Action");

        menu.setMnemonic('a');

        JMenuItem send = new JMenuItem("Send");
        send.setMnemonic('s');
        send.setActionCommand(ACTION_SEND_ACTION);
        send.addActionListener(this);

        menu.add(send);

        JMenuItem cancel = new JMenuItem("Cancel");
        cancel.setMnemonic('c');
        cancel.setActionCommand(ACTION_CANCEL_ACTION);
        cancel.addActionListener(this);

        menu.add(cancel);

        return menu;
    }

    /**
     * Builds the attachments menu.
     *
     * @return The attachments menu.
     */
    private JMenu buildAttachmentMenu() {
        JMenu menu = new JMenu("Attachments");

        menu.setMnemonic('o');

        JMenuItem attach = new JMenuItem("Add Attachment");
        attach.setMnemonic('a');
        attach.setActionCommand(ATTACHMENT_ADD_ACTION);
        attach.addActionListener(this);

        menu.add(attach);

        JMenu remove = new JMenu("Remove Attachment");
        this.attachments = remove;

        menu.add(remove);

        return menu;
    }

    /**
     * Called when send is selected from the action menu.
     */
    private void actionSendAction() {
        this.parent.sendButtonAction();
    }

    /**
     * Called when cancel is selected from the action menu.
     */
    private void actionCancelAction() {
        this.parent.cancelButtonAction();
    }

    /**
     * Called when add is selected from the attachment menu.
     */
    private void attachmentAddAction() {
        this.parent.addAttachmentAction();
    }

    /**
     * Called when an attachment is selected from the remove attachment
     * menu.
     *
     * @param name The name of the attachment.
     */
    private void attachmentRemoveAction(String name) {
        this.parent.removeAttachmentAction(name);
    }

    /**
     * The registered callback for the menu commands.
     *
     * @param event The event that triggered this callback.
     */
    public void actionPerformed(ActionEvent event) {
        String action = event.getActionCommand();

        if (action.equals(ACTION_SEND_ACTION)) {
            actionSendAction();
        } else if (action.equals(ACTION_CANCEL_ACTION)) {
            actionCancelAction();
        } else if (action.equals(ATTACHMENT_ADD_ACTION)) {
            attachmentAddAction();
        } else {
            boolean found = false;

            for (int i = 0; i < attachments.getItemCount(); i++) {
                JMenuItem item = attachments.getItem(i);

                if (action.equals(item.getText())) {
                    i = attachments.getItemCount();
                    found = true;

                    removeAttachment(item);
                    attachmentRemoveAction(action);
                }
            }

            if (!found) {
                EventLogger.getInstance().logError("Invalid action command " +
                                                   action +
                                                   " in ComposeMenu.actionPerformed()...");
            }
        }
    }

    /**
     * Returns a String representation of this ComposeMenu.
     *
     * @return A String representation.
     */
    public String toString() {
        return "ComposeMenu[action menu=" + this.actionMenu +
               ", attachment menu=" + this.attachmentMenu + "]";
    }
}
