/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.controller;

import java.awt.Toolkit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;

import nan.NaN;

import nan.tools.EventLogger;

import nan.view.View;

/**
 * The main menu for the View.
 *
 * @author John David Ratliff
 * @version 1.20, 1/10/04
 */
public class Menu extends JMenuBar implements ActionListener {
    private View view;
    private JMenu fileMenu = buildFileMenu();
    private JMenu preferencesMenu = buildPreferencesMenu();
    private JMenu newsgroupMenu = buildNewsgroupMenu();
    private JMenu messageMenu = buildMessageMenu();
    private JMenu helpMenu = buildHelpMenu();
    private static final int DEFAULT_ACCELERATOR = Toolkit.getDefaultToolkit()
                                                          .getMenuShortcutKeyMask();
    private static final String FILE_EXIT_ACTION = "_file_exit";
    private static final String PREFERENCES_USER_ACTION = "_preferences_user";
    private static final String PREFERENCES_SERVER_ACTION = "_preferences_server";
    private static final String PREFERENCES_NEWSGROUPS_ACTION = "_preferences_newsgroups";
    private static final String NEWSGROUP_MESSAGES_ACTION = "_newsgroup_messages";
    private static final String NEWSGROUP_DOWNLOAD_ACTION = "_newsgroup_download";
    private static final String NEWSGROUP_REFRESH_NEWSGROUP_ACTION = "_newsgroup_refresh_newsgroup";
    private static final String NEWSGROUP_REFRESH_LIST_ACTION = "_newsgroup_refresh_list";
    private static final String MESSAGE_POST_ACTION = "_message_post";
    private static final String MESSAGE_REPLY_ACTION = "_message_reply";
    private static final String MESSAGE_SOURCE_ACTION = "_message_source";
    private static final String HELP_HELP_ACTION = "_help_help";
    private static final String HELP_LICENSE_GPL_ACTION = "_help_license_gpl";
    private static final String HELP_LICENSE_END_USER_ACTION = "_help_license_end_user";
    private static final String HELP_LICENSE_SOURCE_ACTION = "_help_license_source";
    private static final String HELP_ABOUT_ACTION = "_help_about";

    /**
     * Creates a new Menu object.
     *
     * @param view The associated View.
     */
    public Menu(View view) {
        this.view = view;

        add(fileMenu);
        add(preferencesMenu);
        add(newsgroupMenu);
        add(messageMenu);
        add(helpMenu);
    }

    /**
     * Builds the file menu.
     *
     * @return The file menu.
     */
    private JMenu buildFileMenu() {
        JMenu menu = new JMenu("File");

        menu.setMnemonic('f');

        JMenuItem exit = new JMenuItem("Exit");
        exit.setMnemonic('x');
        exit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,
                                                   DEFAULT_ACCELERATOR |
                                                   InputEvent.ALT_DOWN_MASK));
        exit.setActionCommand(FILE_EXIT_ACTION);
        exit.addActionListener(this);

        menu.add(exit);

        return menu;
    }

    /**
     * Builds the preferences menu.
     *
     * @return The preferences menu.
     */
    private JMenu buildPreferencesMenu() {
        JMenu menu = new JMenu("Preferences");

        menu.setMnemonic('p');

        JMenuItem user = new JMenuItem("User Settings");
        user.setMnemonic('u');
        user.setActionCommand(PREFERENCES_USER_ACTION);
        user.addActionListener(this);

        menu.add(user);

        JMenuItem server = new JMenuItem("Server Settings");
        server.setMnemonic('s');
        server.setActionCommand(PREFERENCES_SERVER_ACTION);
        server.addActionListener(this);

        menu.add(server);
        menu.addSeparator();

        JMenuItem newsgroups = new JMenuItem("Newsgroups");
        newsgroups.setMnemonic('n');
        newsgroups.setActionCommand(PREFERENCES_NEWSGROUPS_ACTION);
        newsgroups.addActionListener(this);

        menu.add(newsgroups);

        return menu;
    }

    /**
     * Builds the Newsgroup menu.
     *
     * @return The Newsgroup menu.
     */
    private JMenu buildNewsgroupMenu() {
        JMenu menu = new JMenu("Newsgroup");

        menu.setMnemonic('n');

        JMenuItem messages = new JMenuItem("Set Messages to Cache");
        messages.setMnemonic('m');
        messages.setActionCommand(NEWSGROUP_MESSAGES_ACTION);
        messages.addActionListener(this);

        menu.add(messages);

        JMenuItem download = new JMenuItem("Download Full Messages");
        download.setMnemonic('d');
        download.setActionCommand(NEWSGROUP_DOWNLOAD_ACTION);
        download.addActionListener(this);

        menu.add(download);
        menu.addSeparator();

        JMenuItem refreshNewsgroup = new JMenuItem("Refresh Current Newsgroup");
        refreshNewsgroup.setMnemonic('c');
        refreshNewsgroup.setActionCommand(NEWSGROUP_REFRESH_NEWSGROUP_ACTION);
        refreshNewsgroup.addActionListener(this);

        menu.add(refreshNewsgroup);

        JMenuItem refreshList = new JMenuItem("Check for New Newsgroups");
        refreshList.setMnemonic('n');
        refreshList.setActionCommand(NEWSGROUP_REFRESH_LIST_ACTION);
        refreshList.addActionListener(this);

        menu.add(refreshList);

        return menu;
    }

    /**
     * Builds the message menu.
     *
     * @return The message menu.
     */
    private JMenu buildMessageMenu() {
        JMenu menu = new JMenu("Message");

        menu.setMnemonic('m');

        JMenuItem post = new JMenuItem("Post");
        post.setMnemonic('p');
        post.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P,
                                                   DEFAULT_ACCELERATOR |
                                                   InputEvent.ALT_DOWN_MASK));
        post.setActionCommand(MESSAGE_POST_ACTION);
        post.addActionListener(this);

        menu.add(post);

        JMenuItem reply = new JMenuItem("Reply");
        reply.setMnemonic('r');
        reply.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R,
                                                    DEFAULT_ACCELERATOR |
                                                    InputEvent.ALT_DOWN_MASK));
        reply.setActionCommand(MESSAGE_REPLY_ACTION);
        reply.addActionListener(this);

        menu.add(reply);
        menu.addSeparator();

        JMenuItem source = new JMenuItem("View Source");
        source.setMnemonic('s');
        source.setActionCommand(MESSAGE_SOURCE_ACTION);
        source.addActionListener(this);

        menu.add(source);

        return menu;
    }

    /**
     * Builds the help menu.
     *
     * @return The help menu.
     */
    private JMenu buildHelpMenu() {
        JMenu menu = new JMenu("Help");

        menu.setMnemonic('h');

        JMenuItem help = new JMenuItem("Help with NaN");
        help.setMnemonic('h');
        help.setActionCommand(HELP_HELP_ACTION);
        help.addActionListener(this);

        menu.add(help);

        JMenu license = new JMenu("License");

        JMenuItem gpl = new JMenuItem("View NaN Free License");
        gpl.setActionCommand(HELP_LICENSE_GPL_ACTION);
        gpl.addActionListener(this);

        license.add(gpl);

        JMenuItem endUser = new JMenuItem("View MindTerm End-User License");
        endUser.setActionCommand(HELP_LICENSE_END_USER_ACTION);
        endUser.addActionListener(this);

        license.add(endUser);

        JMenuItem source = new JMenuItem("View MindTerm Public Source License");
        source.setActionCommand(HELP_LICENSE_SOURCE_ACTION);
        source.addActionListener(this);

        license.add(source);

        menu.add(license);
        menu.addSeparator();

        JMenuItem about = new JMenuItem("About");
        about.setMnemonic('a');
        about.setActionCommand(HELP_ABOUT_ACTION);
        about.addActionListener(this);

        menu.add(about);

        return menu;
    }

    /**
     * Called when exit is selected from the file menu.
     */
    private void fileExitAction() {
        this.view.exit(false);
    }

    /**
     * Called when user settings is selected from the preferences menu.
     */
    private void preferencesUserAction() {
        this.view.showUserSettingsDialog();
    }

    /**
     * Called when server settings is selected from the preferences menu.
     */
    private void preferencesServerAction() {
        this.view.showServerSettingsDialog();
    }

    /**
     * Called when newsgroups is selected from the preferences menu.
     */
    private void preferencesNewsgroupsAction() {
        this.view.showNewsgroupDialog();
    }

    /**
     * Called when messages is selected from the newsgroup menu.
     */
    private void newsgroupMessagesAction() {
        this.view.changeNewsgroupMessages();
    }

    /**
     * Called when download is selected from the newsgroup menu.
     */
    private void newsgroupDownloadAction() {
        this.view.cacheCurrentNewsgroup();
    }

    /**
     * Called when refresh newsgroup is selected from the newsgroup menu.
     */
    private void newsgroupRefreshNewsgroupAction() {
        this.view.refreshCurrentNewsgroup();
    }

    /**
     * Called when refresh newsgroup list is selected from the newsgroup
     * menu.
     */
    private void newsgroupRefreshListAction() {
        this.view.refreshNewsgroupList();
    }

    /**
     * Called when post is selected from the message menu.
     */
    private void messagePostAction() {
        this.view.postMessage();
    }

    /**
     * Called when reply is selected from the message menu.
     */
    private void messageReplyAction() {
        this.view.replyToMessage();
    }

    /**
     * Called when view source is selected from the message menu.
     */
    private void messageSourceAction() {
        this.view.showCurrentMessageSource();
    }

    /**
     * Called when Help is selected from the help menu.
     */
    private void helpHelpAction() {
        this.view.showHelp();
    }

    /**
     * Called when View GPL is selected from the help menu.
     */
    private void helpLicenseGPLAction() {
        this.view.displayLicense(NaN.FREE_LICENSE);
    }

    /**
     * Called when View MindTerm End-User is selected from the help menu.
     */
    private void helpLicenseEndUserAction() {
        this.view.displayLicense(NaN.MINDTERM_END_USER_LICENSE);
    }

    /**
     * Called when View MindTerm Source License is selected from the help
     * menu.
     */
    private void helpLicenseSourceAction() {
        this.view.displayLicense(NaN.MINDTERM_SOURCE_LICENSE);
    }

    /**
     * Called when about is selected from the help menu.
     */
    private void helpAboutAction() {
        String title = "About " + NaN.APP_NAME + "...";
        String message = NaN.APP_NAME + ' ' + NaN.APP_VERSION + '\n' +
                         NaN.APP_COPYRIGHT + '\n' + '\n' + "Source has " +
                         NaN.FILE_COUNT + " files and " + NaN.LINE_COUNT +
                         " lines total.";

        JOptionPane.showMessageDialog(view, message, title,
                                      JOptionPane.INFORMATION_MESSAGE);
    }

    /**
     * Listens to the menu selections and calls the appropriate handler.
     *
     * @param event The associated ActionEvent.
     */
    public void actionPerformed(ActionEvent event) {
        String action = event.getActionCommand();

        if (action.equals(FILE_EXIT_ACTION)) {
            fileExitAction();
        } else if (action.equals(PREFERENCES_USER_ACTION)) {
            preferencesUserAction();
        } else if (action.equals(PREFERENCES_SERVER_ACTION)) {
            preferencesServerAction();
        } else if (action.equals(PREFERENCES_NEWSGROUPS_ACTION)) {
            preferencesNewsgroupsAction();
        } else if (action.equals(NEWSGROUP_MESSAGES_ACTION)) {
            newsgroupMessagesAction();
        } else if (action.equals(NEWSGROUP_DOWNLOAD_ACTION)) {
            newsgroupDownloadAction();
        } else if (action.equals(NEWSGROUP_REFRESH_NEWSGROUP_ACTION)) {
            newsgroupRefreshNewsgroupAction();
        } else if (action.equals(NEWSGROUP_REFRESH_LIST_ACTION)) {
            newsgroupRefreshListAction();
        } else if (action.equals(MESSAGE_POST_ACTION)) {
            messagePostAction();
        } else if (action.equals(MESSAGE_REPLY_ACTION)) {
            messageReplyAction();
        } else if (action.equals(MESSAGE_SOURCE_ACTION)) {
            messageSourceAction();
        } else if (action.equals(HELP_HELP_ACTION)) {
            helpHelpAction();
        } else if (action.equals(HELP_LICENSE_GPL_ACTION)) {
            helpLicenseGPLAction();
        } else if (action.equals(HELP_LICENSE_END_USER_ACTION)) {
            helpLicenseEndUserAction();
        } else if (action.equals(HELP_LICENSE_SOURCE_ACTION)) {
            helpLicenseSourceAction();
        } else if (action.equals(HELP_ABOUT_ACTION)) {
            helpAboutAction();
        } else {
            // error. invalid action
            EventLogger.getInstance().logError("nan.controller.Menu: " +
                                               "invalid menu action command " +
                                               action);
        }
    }

    /**
     * Returns a String representation of this Menu.
     *
     * @return A String representation.
     */
    public String toString() {
        return "Menu[file menu=" + this.fileMenu + ", preferences menu=" +
               this.preferencesMenu + ", newsgroup menu=" +
               this.newsgroupMenu + ", message menu=" + this.messageMenu +
               ", help menu=" + this.helpMenu + "]";
    }
}
