/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.model;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import nan.tools.Encoder;
import nan.tools.EventLogger;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

/**
 * An NNTP Message attachment.
 *
 * @author John David Ratliff
 * @version 1.2, 1/5/04
 */
public class Attachment implements XMLSerializable {
    private String name;
    private String data;

    /**
     * XML tag for serialization.
     */
    public static final String XML_TAG = "attachment";

    /**
     * XML tag for name property.
     */
    public static final String XML_NAME = "name";

    /**
     * XML tag for data property.
     */
    public static final String XML_DATA = "data";

    /**
     * Creates a new Attachment object.
     *
     * @param name The name of the attachment.
     * @param data The Base64 encoded data.
     */
    public Attachment(String name, String data) {
        this.name = name;
        this.data = data;
    }

    /**
     * Creates a new Attachment object.
     */
    protected Attachment() {
    }

    /**
     * Gets the name of this attachment.
     *
     * @return The name.
     */
    public String getName() {
        return this.name;
    }

    /**
     * Sets the name of this attachment.
     *
     * @param name The name.
     */
    protected void setName(String name) {
        this.name = name;
    }

    /**
     * Gets the Base64 encoded data of this attachment.
     *
     * @return The Base64 encoded data.
     */
    public String getData() {
        return this.data;
    }

    /**
     * Sets the Base64 encoded data of this attachment.
     *
     * @param data The Base64 encoded data.
     */
    protected void setData(String data) {
        this.data = data;
    }

    /**
     * Returns this Attachment as a UUEncoded document.
     *
     * @return The UUE String.
     */
    public String getUUEData() {
        StringBuffer buffer = new StringBuffer();

        buffer.append("begin 644 ");
        buffer.append(getName());
        buffer.append('\n');

        buffer.append(getData());

        buffer.append("end");
        buffer.append('\n');

        return buffer.toString();
    }

    /**
     * Decodes and returns the data for this attachment.
     *
     * @return The unencoded data.
     */
    public byte[] decode() {
        return Encoder.uudecode(getData());
    }

    /**
     * Encodes a file to an Attachment.
     *
     * @param file The file to encode.
     *
     * @return The Attachment.
     */
    public static Attachment encode(File file) {
        try {
            FileInputStream input = new FileInputStream(file);
            byte[] data = new byte[(int)file.length()];
            byte[] buffer = new byte[8192];

            int offset = 0;
            int length;

            while ((length = input.read(buffer, 0, 8192)) > 0) {
                System.arraycopy(buffer, 0, data, offset, length);
                offset += length;
            }

            input.close();

            return new Attachment(file.getName(), Encoder.uuencode(data));
        } catch (FileNotFoundException e) {
            EventLogger.getInstance().logError("FileNotFoundException during Attachment.encode()...");
        } catch (IOException e) {
            EventLogger.getInstance().logError("IOException during Attachment.encode()...");
        }

        return null;
    }

    /**
     * Converts an object into an XML Document.
     *
     * @return The XML Document.
     */
    public Document toXML() {
        Document document = DocumentHelper.createDocument();

        // the root element
        Element root = document.addElement(XML_TAG);

        // the attachment properties
        Element name = root.addElement(XML_NAME);
        name.setText(getName());

        Element data = root.addElement(XML_DATA);
        data.setText(getData());

        return document;
    }

    /**
     * Returns a String representation of this attachment.
     *
     * @return A String representation.
     */
    public String toString() {
        return "Attachment[name=" + getName() + "]";
    }
}
