/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.model;

import java.util.Date;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

/**
 * A representation of a License agreement.
 *
 * @author John David Ratliff
 * @version 1.2, 1/2/04
 */
public class License implements XMLSerializable, Transposer {
    private String name;
    private boolean accepted;
    private Date date;

    /**
     * XML tag used for serialization.
     */
    public static final String XML_TAG = "license";

    /**
     * XML tag for the name property.
     */
    public static final String XML_NAME = "name";

    /**
     * XML tag for the acceptance property.
     */
    public static final String XML_ACCEPTED = "accepted";

    /**
     * XML tag for the date property.
     */
    public static final String XML_DATE = "date";

    /**
     * Creates a new License object.
     *
     * @param name The name of the license.
     * @param accepted Whether the license has been accepted or not.
     * @param date The date of the agreement.
     */
    public License(String name, boolean accepted, Date date) {
        this.name = name;
        this.accepted = accepted;
        this.date = date;
    }

    /**
     * Creates a new License object.
     */
    protected License() {
    }

    /**
     * Gets the name of this License.
     *
     * @return The name.
     */
    public String getName() {
        return this.name;
    }

    /**
     * Sets the name of this License.
     *
     * @param name The name.
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Asks if this License has been accepted or not.
     *
     * @return true if the License is accepted; false otherwise.
     */
    public boolean wasAccepted() {
        return this.accepted;
    }

    /**
     * Sets if this License has been accepted or not.
     *
     * @param accepted true if the License is accepted; false otherwise.
     */
    public void setAccepted(boolean accepted) {
        this.accepted = accepted;
    }

    /**
     * Gets the date of this license agreement.
     *
     * @return The date.
     */
    public Date getDate() {
        return date;
    }

    /**
     * Sets the date of this license agreement.
     *
     * @param date The date.
     */
    public void setDate(Date date) {
        this.date = date;
    }

    /**
     * Converts an object into an XML Document.
     *
     * @return The XML Document.
     */
    public Document toXML() {
        Document document = DocumentHelper.createDocument();

        // the root element
        Element root = document.addElement(XML_TAG);

        // the properties
        Element name = root.addElement(XML_NAME);
        name.setText(getName());

        Element accepted = root.addElement(XML_ACCEPTED);
        accepted.setText("" + wasAccepted());

        Element date = root.addElement(XML_DATE);
        date.setText("" + getDate().getTime());

        return document;
    }

    /**
     * Transposes (copies over) the values of another object of the same
     * type onto this one. If the object is not the same type as the
     * invoked object, it will be ignored.
     *
     * @param object The object to transpose over this one.
     */
    public void transpose(Object object) {
        if (object instanceof License) {
            License license = (License)object;

            this.name = license.name;
            this.accepted = license.accepted;
            this.date = license.date;
        }
    }

    /**
     * Returns a String representation of this license.
     *
     * @return A String representation.
     */
    public String toString() {
        return "License[name=" + getName() + ", accepted=" + wasAccepted() +
               ", date=" + getDate() + "]";
    }
}
