/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.model;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import nan.exception.NoSuchHeaderException;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

/**
 * A representation of an NNTP message header in RFC 850 format.
 *
 * @author John David Ratliff
 * @version 1.12, 1/2/04
 */
public class MessageHeader implements XMLSerializable,
                                      Transposer,
                                      Selected {
    private Map headers;
    private boolean read;

    /**
     * XML tag for serialization purposes.
     */
    public static final String XML_TAG = "message-header";

    /**
     * XML tag for header property.
     */
    public static final String XML_HEADER = "header";

    /**
     * XML tag for header name.
     */
    public static final String XML_NAME = "name";

    /**
     * XML tag for header value.
     */
    public static final String XML_VALUE = "value";

    /**
     * XML tag for read property.
     */
    public static final String XML_READ = "read";

    /**
     * Creates a new MessageHeader object.
     */
    public MessageHeader() {
        this.headers = new Hashtable();
    }

    /**
     * Adds a header to this MessageHader. Duplicate headers are ignored. To
     * change a value, use the replaceHeader() method.
     *
     * @param name The name of the header. Case is ignored.
     * @param value The value of the header.
     */
    public void addHeader(String name, String value) {
        String key = name.toLowerCase();

        value = name + ": " + value;

        if (!headers.containsKey(key)) {
            headers.put(key, value);
        }
    }

    /**
     * Gets a message header.
     *
     * @param name The name of the header. Case is ignored.
     *
     * @return The retrieved header.
     *
     * @throws NoSuchHeaderException if the header does not exist.
     */
    public String getHeader(String name) throws NoSuchHeaderException {
        String key = name.toLowerCase();

        if (headers.containsKey(key)) {
            return (String)headers.get(key);
        }

        throw new NoSuchHeaderException();
    }

    /**
     * Gets the value of a header.
     *
     * @param name The name of the header. Case is ignored.
     *
     * @return The header value.
     *
     * @throws NoSuchHeaderException if the header does not exist.
     */
    public String getValue(String name) throws NoSuchHeaderException {
        String header = getHeader(name);
        int index = header.indexOf(':');

        return header.substring(index + 2);
    }

    /**
     * Replaces a header.
     *
     * @param name The name of the header. Case is ignored.
     * @param value The value of the header.
     *
     * @throws NoSuchHeaderException if the header does not exist.
     */
    public void replaceHeader(String name, String value)
                       throws NoSuchHeaderException {
        String key = name.toLowerCase();

        if (!headers.containsKey(key)) {
            throw new NoSuchHeaderException();
        }

        headers.remove(key);
        addHeader(name, value);
    }

    /**
     * Asks if this MessageHeader contains a particular header.
     *
     * @param name The name of the header. Case is ignored.
     *
     * @return true if the header exists; false otherwise.
     */
    public boolean containsHeader(String name) {
        String key = name.toLowerCase();

        return (this.headers.containsKey(key));
    }

    /**
     * Gets a String list of all the headers.
     *
     * @return The headers.
     */
    public String getHeaders() {
        StringBuffer buffer = new StringBuffer();
        Iterator iterator = this.headers.values().iterator();

        while (iterator.hasNext()) {
            String header = (String)iterator.next();
            buffer.append(header);
            buffer.append('\n');
        }

        return buffer.toString();
    }

    /**
     * Asks if this message has been read.
     *
     * @return true if this message has been read; false otherwise.
     */
    public boolean isRead() {
        return this.read;
    }

    /**
     * Sets if this message has been read.
     *
     * @param read true if read; false otherwise.
     */
    public void setRead(boolean read) {
        this.read = read;
    }

    /**
     * Converts an object into an XML Document.
     *
     * @return The XML Document.
     */
    public Document toXML() {
        Document document = DocumentHelper.createDocument();

        // the root element
        Element root = document.addElement(XML_TAG);

        Element read = root.addElement(XML_READ);
        read.setText("" + isRead());

        Iterator iterator = this.headers.values().iterator();

        while (iterator.hasNext()) {
            String header = (String)iterator.next();

            int colon = header.indexOf(':');

            String name = header.substring(0, colon);
            String value = header.substring(colon + 2);

            Element element = root.addElement(XML_HEADER);

            Element xmlName = element.addElement(XML_NAME);
            xmlName.setText(name);

            Element xmlValue = element.addElement(XML_VALUE);
            xmlValue.setText(value);
        }

        return document;
    }

    /**
     * Transposes (copies over) the values of another object of the same
     * type onto this one. If the object is not the same type as the
     * invoked object, it will be ignored.
     *
     * @param object The object to transpose over this one.
     */
    public void transpose(Object object) {
        if (object instanceof MessageHeader) {
            MessageHeader header = (MessageHeader)object;

            this.headers = header.headers;
            this.read = header.read;
        }
    }

    /**
     * Returns a String representation of this MessageHeader. Good for
     * JTreeTable display cells.
     *
     * @return A String representation.
     */
    public String toString() {
        String from = getValue("from");

        int index = from.lastIndexOf(' ');

        if (index > 0) {
            from = from.substring(0, index);
        }

        if (from.charAt(0) == '"') {
            from = from.substring(1, from.length() - 1);
        }

        return from;
    }
}
