/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.model;

import java.util.Enumeration;
import java.util.StringTokenizer;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;

import nan.tools.EventLogger;

/**
 * A tree for Messages.
 *
 * @author John David Ratliff
 * @version 1.8, 1/2/04
 */
public class MessageTree implements Transposer {
    private DefaultMutableTreeNode root;

    /**
     * Creates a new MessageTree object.
     */
    public MessageTree() {
        this.root = new DefaultMutableTreeNode();
    }

    /**
     * Gets the root of this Tree. Useful in components like JTree.
     *
     * @return The TreeNode root.
     */
    public TreeNode getRoot() {
        return this.root;
    }

    /**
     * Returns an enumeration containing all the children in a pre-order
     * traversal pattern.
     *
     * @return The child Enumeration.
     */
    public Enumeration preorderTraversal() {
        return this.root.preorderEnumeration();
    }

    /**
     * Adds a child to this tree just under the root.
     *
     * @param object The user object to encase in the tree node.
     */
    private void addChild(Object object) {
        DefaultMutableTreeNode node = new DefaultMutableTreeNode(object);
        root.add(node);
    }

    /**
     * Adds a child to this tree just under the root.
     *
     * @param message The Message to encapsulate.
     */
    public void addChild(Message message) {
        addChild((Object)message);
    }

    /**
     * Adds a child to this tree just under the root.
     *
     * @param header The MessageHeader to encapsulate.
     */
    public void addChild(MessageHeader header) {
        addChild((Object)header);
    }

    /**
     * Adds a thread child in the proper sequence to this tree.
     *
     * @param threadMessage The message to add.
     * @param references The path to the correct thread position.
     */
    private void addThreadChild(Object threadMessage,
                                StringTokenizer references) {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)getRoot();

        while (references.hasMoreTokens()) {
            boolean foundChild = false;
            String reference = references.nextToken();

            for (int i = 0; i < node.getChildCount(); i++) {
                DefaultMutableTreeNode child = (DefaultMutableTreeNode)node.getChildAt(i);
                Object object = child.getUserObject();

                MessageHeader header = null;

                if (object instanceof Message) {
                    Message message = (Message)object;
                    header = message.getHeader();
                } else if (object instanceof MessageHeader) {
                    header = (MessageHeader)object;
                } else {
                    EventLogger.getInstance().logError("User object is not of type Message or " +
                                                       "MessageHeader in " +
                                                       "MessageTree.addThreadChild()...");
                }

                String messageId = header.getValue("message-id");

                if (reference.equals(messageId)) {
                    // we found the right child
                    // continue downward search
                    foundChild = true;
                    i = node.getChildCount();
                    node = child;
                }
            }

            if (!foundChild) {
                // orphan message...
                // no point continuing... just add it here...
                break;
            }
        }

        // we are out of references, or children, or something. we're here anyway
        // this should be the right place to add the child
        DefaultMutableTreeNode child = new DefaultMutableTreeNode(threadMessage);
        node.add(child);
    }

    /**
     * Adds a thread child in the proper sequence to this tree.
     *
     * @param message The Message to add.
     * @param references The path to the correct thread position.
     */
    public void addThreadChild(Message message, StringTokenizer references) {
        addThreadChild((Object)message, references);
    }

    /**
     * Adds a thread child in the proper sequence to this tree.
     *
     * @param header The MessageHeader to add.
     * @param references The path to the correct thread position.
     */
    public void addThreadChild(MessageHeader header,
                               StringTokenizer references) {
        addThreadChild((Object)header, references);
    }

    /**
     * Transposes (copies over) the values of another object of the same
     * type onto this one. If the object is not the same type as the
     * invoked object, it will be ignored.
     *
     * @param object The object to transpose over this one.
     */
    public void transpose(Object object) {
        if (object instanceof MessageTree) {
            MessageTree tree = (MessageTree)object;

            this.root = tree.root;
        }
    }

    /**
     * Returns a String representation of this MessageTree.
     *
     * @return A String representation.
     */
    public String toString() {
        return "MessageTree[root=" + this.root + "]";
    }
}
