/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.model;

import java.util.Date;

import nan.model.MessageTree;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

/**
 * A representation of an NNTP newsgroup.
 *
 * @author John David Ratliff
 * @version 1.12, 1/2/04
 */
public class Newsgroup implements XMLSerializable,
                                  Comparable,
                                  Transposer {
    private String name;
    private int firstArticle;
    private int lastArticle;
    private int nextArticle;
    private boolean postingAllowed;
    private MessageTree cache;
    private Date updated;
    private int messages;

    /**
     * XML tag for serialization.
     */
    public static final String XML_TAG = "newsgroup";

    /**
     * XML tag for the name property.
     */
    public static final String XML_NAME = "name";

    /**
     * XML tag for the first article property.
     */
    public static final String XML_FIRST_ARTICLE = "first-article";

    /**
     * XML tag for the last article property.
     */
    public static final String XML_LAST_ARTICLE = "last-article";

    /**
     * XML tag for the next article property.
     */
    public static final String XML_NEXT_ARTICLE = "next-article";

    /**
     * XML tag for the posting allowed property.
     */
    public static final String XML_POSTING_ALLOWED = "posting-allowed";

    /**
     * XML tag for the updated property.
     */
    public static final String XML_UPDATED = "updated";

    /**
     * XML tag for the messages property.
     */
    public static final String XML_MESSAGES = "messages";

    /**
     * The default number of messages to cache locally.
     */
    public static final int DEFAULT_MESSAGES = 50;

    /**
     * Integer constant representing that all messages should be cached
     * locally.
     */
    public static final int ALL_MESSAGES = 0;

    /**
     * Creates a new Newsgroup object.
     *
     * @param name The name of this Newsgroup.
     * @param postingAllowed Whether this Newsgroup permits posting.
     * @param messages The number of messages to cache locally (0 = all).
     */
    public Newsgroup(String name, boolean postingAllowed, int messages) {
        this.name = name;
        this.postingAllowed = postingAllowed;
        this.updated = new Date(0);
        this.messages = messages;
    }

    /**
     * Creates a new Newsgroup object.
     */
    protected Newsgroup() {
    }

    /**
     * Gets the name of this Newsgroup.
     *
     * @return The name.
     */
    public String getName() {
        return this.name;
    }

    /**
     * Sets the name of this Newsgroup.
     *
     * @param name The name.
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Gets the first article id in this Newsgroup.
     *
     * @return The first article id.
     */
    public int getFirstArticle() {
        return this.firstArticle;
    }

    /**
     * Sets the first article id in this Newsgroup.
     *
     * @param firstArticle The first article id.
     */
    public void setFirstArticle(int firstArticle) {
        this.firstArticle = firstArticle;
    }

    /**
     * Gets the last article id in this Newsgroup.
     *
     * @return The last article id.
     */
    public int getLastArticle() {
        return this.lastArticle;
    }

    /**
     * Sets the last article id in this Newsgroup.
     *
     * @param lastArticle The last article id.
     */
    public void setLastArticle(int lastArticle) {
        this.lastArticle = lastArticle;
    }

    /**
     * Gets the next article in need of retrieval.
     *
     * @return The next article number.
     */
    public int getNextArticle() {
        return this.nextArticle;
    }

    /**
     * Sets the next article in need of retrieval.
     *
     * @param nextArticle The next article number.
     */
    public void setNextArticle(int nextArticle) {
        this.nextArticle = nextArticle;
    }

    /**
     * Asks if this Newsgroup permits posting.
     *
     * @return true if posting is allowed; false otherwise.
     */
    public boolean isPostingAllowed() {
        return this.postingAllowed;
    }

    /**
     * Sets whether posting is allowed on this Newsgroup or not.
     *
     * @param postingAllowed true if posting is allowed; false otherwise.
     */
    public void setPostingAllowed(boolean postingAllowed) {
        this.postingAllowed = postingAllowed;
    }

    /**
     * Asks if this Newsgroup has a current cache.
     *
     * @return true if this Newsgroup has a local cache; false otherwise.
     */
    public boolean hasCache() {
        return (this.cache != null);
    }

    /**
     * Gets the local cache.
     *
     * @return The MessageTree cache.
     */
    public MessageTree getCache() {
        return this.cache;
    }

    /**
     * Sets the local cache.
     *
     * @param cache The MessageTree cache.
     */
    public void setCache(MessageTree cache) {
        this.cache = cache;
    }

    /**
     * Sets the last update time.
     *
     * @param date The new Date.
     */
    protected void setUpdated(Date date) {
        this.updated = date;
    }

    /**
     * Sets the current time as the last update time.
     */
    public void setUpdated() {
        this.updated = new Date();
    }

    /**
     * Gets the last update time.
     *
     * @return The Date of the last update.
     */
    public Date getUpdated() {
        return this.updated;
    }

    /**
     * Gets the number of messages to cache locally.
     *
     * @return The number of messages.
     */
    public int getMessages() {
        return this.messages;
    }

    /**
     * Sets the number of messages to cache locally.
     *
     * @param messages The number of messages.
     */
    public void setMessages(int messages) {
        this.messages = messages;
    }

    /**
     * Converts an object into an XML Document.
     *
     * @return The XML Document.
     */
    public Document toXML() {
        Document document = DocumentHelper.createDocument();

        // add the root element
        Element root = document.addElement(XML_TAG);

        // add the newsgroup properties
        Element name = root.addElement(XML_NAME);
        name.setText(getName());

        Element firstArticle = root.addElement(XML_FIRST_ARTICLE);
        firstArticle.setText("" + getFirstArticle());

        Element lastArticle = root.addElement(XML_LAST_ARTICLE);
        lastArticle.setText("" + getLastArticle());

        Element nextArticle = root.addElement(XML_NEXT_ARTICLE);
        nextArticle.setText("" + getNextArticle());

        Element postingAllowed = root.addElement(XML_POSTING_ALLOWED);
        postingAllowed.setText("" + isPostingAllowed());

        Element updated = root.addElement(XML_UPDATED);
        updated.setText("" + getUpdated().getTime());

        Element messages = root.addElement(XML_MESSAGES);
        messages.setText("" + getMessages());

        return document;
    }

    /**
     * Compares this Newsgroup to another object for lexiographic
     * equivalence by Newsgroup name.
     *
     * @param object The object to compare with.
     *
     * @return 0 if the objects are equal or not of the same type. Less than
     *         0 if this group comes alphabetically before the object, and
     *         greater than 1 if this group comes alphabetically after the
     *         object.
     */
    public int compareTo(Object object) {
        if (object instanceof Newsgroup) {
            Newsgroup newsgroup = (Newsgroup)object;

            return getName().compareTo(newsgroup.getName());
        }

        return 0;
    }

    /**
     * Tests for equivalence with another object. Two Newsgroup objects are
     * said to be equivalent if their names are the same (String equality,
     * case sensitive).
     *
     * @param object The other object to test equivalence with.
     *
     * @return true if the two objects are equivalent; false otherwise.
     */
    public boolean equals(Object object) {
        if (object instanceof Newsgroup) {
            Newsgroup newsgroup = (Newsgroup)object;

            return newsgroup.getName().equals(getName());
        }

        return false;
    }

    /**
     * Transposes (copies over) the values of another object of the same
     * type onto this one. If the object is not the same type as the
     * invoked object, it will be ignored.
     *
     * @param object The object to transpose over this one.
     */
    public void transpose(Object object) {
        if (object instanceof Newsgroup) {
            Newsgroup newsgroup = (Newsgroup)object;

            this.name = newsgroup.name;
            this.firstArticle = newsgroup.firstArticle;
            this.lastArticle = newsgroup.lastArticle;
            this.nextArticle = newsgroup.nextArticle;
            this.postingAllowed = newsgroup.postingAllowed;
            this.updated = newsgroup.updated;
            this.messages = newsgroup.messages;
        }
    }

    /**
     * Returns a String representation of this Newsgroup.
     *
     * @return A String representation.
     */
    public String toString() {
        return this.getName();
    }
}
