/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.ListModel;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

/**
 * A list container for Newsgroups that also support JList components.
 *
 * @author John David Ratliff
 * @version 1.11, 1/2/04
 */
public class NewsgroupList implements XMLSerializable, Transposer {
    private DefaultListModel model;
    private Date updated;

    /**
     * XML tag for serialization.
     */
    public static final String XML_TAG = "newsgroup-list";

    /**
     * XML tag for the last updated property.
     */
    public static final String XML_UPDATED = "last-updated";

    /**
     * Creates a new NewsgroupList object.
     */
    public NewsgroupList() {
        this.model = new DefaultListModel();
        this.updated = new Date();
    }

    /**
     * Appends the Newsgroups from a NewsgroupList onto this NewsgroupList.
     *
     * @param list The NewsgroupList to append from.
     */
    public void append(NewsgroupList list) {
        for (int i = 0; i < list.getSize(); i++) {
            Newsgroup newsgroup = list.get(i);
            add(newsgroup);
        }
    }

    /**
     * Gets the number of Newsgroups in this NewsgroupList.
     *
     * @return The Newsgroup count.
     */
    public int getSize() {
        return this.model.getSize();
    }

    /**
     * Asks if a Newsgroup is contained in this list.
     *
     * @param newsgroup The Newsgroup to check.
     *
     * @return true if the Newsgroup is in this list; false otherwise.
     */
    public boolean contains(Newsgroup newsgroup) {
        return this.model.contains(newsgroup);
    }

    /**
     * Adds a Newsgroup to this list.
     *
     * @param newsgroup The Newsgroup.
     */
    public void add(Newsgroup newsgroup) {
        if (!contains(newsgroup)) {
            this.model.addElement(newsgroup);
        }
    }

    /**
     * Removes a Newsgroup from this list.
     *
     * @param index The index to remove at.
     *
     * @return The removed Newsgroup.
     *
     * @throws IndexOutOfBoundsException if the index is invalid.
     */
    public Newsgroup remove(int index) throws IndexOutOfBoundsException {
        if ((index >= 0) && (index < this.model.getSize())) {
            return (Newsgroup)this.model.remove(index);
        } else {
            throw new IndexOutOfBoundsException();
        }
    }

    /**
     * Gets a Newsgroup from this list.
     *
     * @param index The index of the Newsgroup to retrieve.
     *
     * @return The retrieved Newsgroup.
     *
     * @throws IndexOutOfBoundsException if the index is invalid.
     */
    public Newsgroup get(int index) throws IndexOutOfBoundsException {
        if ((index >= 0) && (index < this.model.getSize())) {
            return (Newsgroup)this.model.get(index);
        }

        throw new IndexOutOfBoundsException();
    }

    /**
     * Sorts this list by Newsgroup.compareTo, which by default is an
     * alpbetical sort.
     */
    public void sort() {
        List list = new ArrayList();

        for (int i = 0; i < this.model.getSize(); i++) {
            list.add(this.model.get(i));
        }

        Collections.sort(list);
        this.model.clear();

        for (int i = 0; i < list.size(); i++) {
            this.model.addElement(list.get(i));
        }
    }

    /**
     * Gets the ListModel. Useful for components like JList.
     *
     * @return The ListModel.
     */
    public ListModel getListModel() {
        return this.model;
    }

    /**
     * Sets the last update time.
     *
     * @param date The new Date.
     */
    protected void setUpdated(Date date) {
        this.updated = date;
    }

    /**
     * Sets the current time as the last update time.
     */
    public void setUpdated() {
        this.updated = new Date();
    }

    /**
     * Gets the last update time.
     *
     * @return The Date of the last update.
     */
    public Date getUpdated() {
        return this.updated;
    }

    /**
     * Converts an object into an XML Document.
     *
     * @return The XML Document.
     */
    public Document toXML() {
        Document document = DocumentHelper.createDocument();

        // the root Element
        Element root = document.addElement(XML_TAG);

        // the updated property
        Element updated = root.addElement(XML_UPDATED);
        updated.setText("" + getUpdated().getTime());

        // the newsgroups...
        for (int i = 0; i < this.getSize(); i++) {
            Newsgroup newsgroup = get(i);

            Document xml = newsgroup.toXML();
            Element element = xml.getRootElement();
            element.setParent(null);

            root.add(element);
        }

        return document;
    }

    /**
     * Transposes (copies over) the values of another object of the same
     * type onto this one. If the object is not the same type as the
     * invoked object, it will be ignored.
     *
     * @param object The object to transpose over this one.
     */
    public void transpose(Object object) {
        if (object instanceof NewsgroupList) {
            NewsgroupList list = (NewsgroupList)object;

            this.model = list.model;
            this.updated = list.updated;
        }
    }

    /**
     * Returns a String representation of this NewsgroupList.
     *
     * @return A String representation.
     */
    public String toString() {
        return "NewsgroupList[model=" + this.model + ", updated=" +
               this.updated + "]";
    }
}
