/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.model;

import nan.exception.InvalidPortException;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

/**
 * The settings used by the News Server.
 *
 * @author John David Ratliff
 * @version 1.8, 1/4/04
 */
public class ServerSettings implements XMLSerializable, Transposer {
    private String newsServer = DEFAULT_NEWS_SERVER;
    private int newsPort = DEFAULT_NEWS_PORT;
    private boolean useSSH;
    private String sshServer = EMPTY;
    private int sshPort = DEFAULT_SSH_PORT;
    private String username = EMPTY;
    private String password = EMPTY;
    private boolean useTunnel;
    private int localPort = DEFAULT_LOCAL_PORT;
    private boolean useRemoteSave;
    private String directory = EMPTY;
    private static final String EMPTY = "";

    /**
     * XML tag for serialization.
     */
    public static final String XML_TAG = "server-settings";

    /**
     * XML tag for news server property.
     */
    public static final String XML_NEWS_SERVER = "news-server";

    /**
     * XML tag for news port property.
     */
    public static final String XML_NEWS_PORT = "news-port";

    /**
     * XML tag for SSH property.
     */
    public static final String XML_USE_SSH = "ssh";

    /**
     * XML tag for ssh server hostname property.
     */
    public static final String XML_SSH_SERVER = "server";

    /**
     * XML tag for ssh server port property.
     */
    public static final String XML_SSH_PORT = "port";

    /**
     * XML tag for ssh username property.
     */
    public static final String XML_SSH_USERNAME = "username";

    /**
     * XML tag for the ssh tunnel property.
     */
    public static final String XML_USE_TUNNEL = "tunnel";

    /**
     * XML tag for ssh local port property.
     */
    public static final String XML_LOCAL_PORT = "local-port";

    /**
     * XML tag for the remote save property.
     */
    public static final String XML_USE_REMOTE_SAVE = "remote-save";

    /**
     * XML tag for the remote save directory property.
     */
    public static final String XML_DIRECTORY = "directory";

    /**
     * The default news server hostname.
     */
    public static final String DEFAULT_NEWS_SERVER = "localhost";

    /**
     * The default news port.
     */
    public static final int DEFAULT_NEWS_PORT = 119;

    /**
     * The default SSH port.
     */
    public static final int DEFAULT_SSH_PORT = 22;

    /**
     * The default local port for SSH tunnels.
     */
    public static final int DEFAULT_LOCAL_PORT = 1119;

    /**
     * Creates a new ServerSettings object.
     */
    public ServerSettings() {
        // the defaults are fine here...
    }

    /**
     * Converts this ServerSettings into an XML Document.
     *
     * @return The XML Document.
     */
    public Document toXML() {
        Document document = DocumentHelper.createDocument();

        // root element
        Element root = document.addElement(XML_TAG);

        // add the settings to the root
        Element newsServer = root.addElement(XML_NEWS_SERVER);
        newsServer.setText(getNewsServer());

        Element newsPort = root.addElement(XML_NEWS_PORT);
        newsPort.setText("" + getNewsPort());

        if (useSSH()) {
            Element ssh = root.addElement(XML_USE_SSH);

            // add the SSH components
            Element sshServer = ssh.addElement(XML_SSH_SERVER);
            sshServer.setText(getSSHServer());

            Element sshPort = ssh.addElement(XML_SSH_PORT);
            sshPort.setText("" + getSSHPort());

            Element username = ssh.addElement(XML_SSH_USERNAME);
            username.setText(getUsername());

            if (useTunnel()) {
                // add the tunnel components
                Element tunnel = ssh.addElement(XML_USE_TUNNEL);

                Element localPort = tunnel.addElement(XML_LOCAL_PORT);
                localPort.setText("" + getLocalPort());
            }

            if (useRemoteSave()) {
                // add the remote save components
                Element remoteSave = ssh.addElement(XML_USE_REMOTE_SAVE);

                Element directory = remoteSave.addElement(XML_DIRECTORY);
                directory.setText(getDirectory());
            }
        }

        return document;
    }

    /**
     * Gets the news server hostname.
     *
     * @return The hostname.
     */
    public String getNewsServer() {
        return this.newsServer;
    }

    /**
     * Sets the new server hostname.
     *
     * @param newsServer The hostname.
     */
    public void setNewsServer(String newsServer) {
        this.newsServer = newsServer;
    }

    /**
     * Gets the news server port.
     *
     * @return The port.
     */
    public int getNewsPort() {
        return this.newsPort;
    }

    /**
     * Sets the news server port.
     *
     * @param port The port.
     *
     * @throws InvalidPortException if the port is out of range (1 &lt; port
     *         &lt; 65535).
     */
    public void setNewsPort(int port) throws InvalidPortException {
        // port must lie within range [1, 65535]
        if ((port < 1) || (port > 65535)) {
            throw new InvalidPortException();
        }

        this.newsPort = port;
    }

    /**
     * If SSH is being used.
     *
     * @return true if using SSH; false otherwise.
     */
    public boolean useSSH() {
        return this.useSSH;
    }

    /**
     * Sets if SSH is being used.
     *
     * @param useSSH true to use SSH; false otherwise.
     */
    public void setUseSSH(boolean useSSH) {
        this.useSSH = useSSH;
    }

    /**
     * Gets the SSH server hostname.
     *
     * @return The hostname.
     */
    public String getSSHServer() {
        return this.sshServer;
    }

    /**
     * Sets the SSH server hostname.
     *
     * @param sshServer The hostname.
     */
    public void setSSHServer(String sshServer) {
        this.sshServer = sshServer;
    }

    /**
     * Gets the SSH port.
     *
     * @return The port.
     */
    public int getSSHPort() {
        return this.sshPort;
    }

    /**
     * Sets the SSH server port.
     *
     * @param port The port.
     *
     * @throws InvalidPortException if the port is out of range (1 &lt; port
     *         &lt; 65535).
     */
    public void setSSHPort(int port) throws InvalidPortException {
        // port must lie within range [1, 65535]
        if ((port < 1) || (port > 65535)) {
            throw new InvalidPortException();
        }

        this.sshPort = port;
    }

    /**
     * Gets the username for the SSH account.
     *
     * @return The SSH username.
     */
    public String getUsername() {
        return this.username;
    }

    /**
     * Sets the SSH username.
     *
     * @param username The username.
     */
    public void setUsername(String username) {
        this.username = username;
    }

    /**
     * Gets the SSH password.
     *
     * @return The password.
     */
    public String getPassword() {
        return this.password;
    }

    /**
     * Sets the password for the SSH account.
     *
     * @param password The SSH password.
     */
    public void setPassword(String password) {
        this.password = password;
    }

    /**
     * Whether an SSH tunnel is being used or not.
     *
     * @return true for SSH tunnel use; false otherwise.
     */
    public boolean useTunnel() {
        return this.useTunnel;
    }

    /**
     * Gets the local port used for the SSH tunnel.
     *
     * @return The port.
     */
    public int getLocalPort() {
        return this.localPort;
    }

    /**
     * Sets the local port used for the SSH tunnel.
     *
     * @param port The port.
     *
     * @throws InvalidPortException if the port is out of range (1024 &lt;
     *         port &lt; 65535).
     */
    public void setLocalPort(int port) throws InvalidPortException {
        // local port must lie within range [1024, 65535]
        if ((port < 1024) || (port > 65535)) {
            throw new InvalidPortException();
        }

        if (useSSH()) {
            this.localPort = port;
            this.useTunnel = true;
        }
    }

    /**
     * Whether remote save is being used or not.
     *
     * @return true if using remote save; false otherwise.
     */
    public boolean useRemoteSave() {
        return this.useRemoteSave;
    }

    /**
     * Gets the directory for remote save.
     *
     * @return The remote save directory.
     */
    public String getDirectory() {
        return this.directory;
    }

    /**
     * Sets the directory for remote save.
     *
     * @param directory The directory.
     */
    public void setDirectory(String directory) {
        if (useSSH()) {
            if (directory != null) {
                this.directory = directory;
                this.useRemoteSave = true;
            } else {
                this.useRemoteSave = false;
            }
        }
    }

    /**
     * Tests if this ServerSettings is logically equivalent to another
     * ServerSettings.
     *
     * @param object The object to test for equality with.
     *
     * @return true if the objects are equal; false otherwise.
     */
    public boolean equals(Object object) {
        if (object instanceof ServerSettings) {
            ServerSettings settings = (ServerSettings)object;

            return ((newsServer.equals(settings.newsServer)) &&
                   (newsPort == settings.newsPort) &&
                   (useSSH == settings.useSSH) &&
                   (sshServer.equals(settings.sshServer)) &&
                   (sshPort == settings.sshPort) &&
                   (username.equals(settings.username)) &&
                   (password.equals(settings.password)) &&
                   (useTunnel == settings.useTunnel) &&
                   (localPort == settings.localPort) &&
                   (useRemoteSave == settings.useRemoteSave) &&
                   (directory.equals(settings.directory)));
        }

        return false;
    }

    /**
     * Transposes (copies over) the values of another object of the same
     * type onto this one. If the object is not the same type as the
     * invoked object, it will be ignored.
     *
     * @param object The object to transpose over this one.
     */
    public void transpose(Object object) {
        if (object instanceof ServerSettings) {
            ServerSettings settings = (ServerSettings)object;

            this.newsServer = settings.newsServer;
            this.newsPort = settings.newsPort;

            this.useSSH = settings.useSSH;

            this.sshServer = settings.sshServer;
            this.sshPort = settings.sshPort;
            this.username = settings.username;
            this.password = settings.password;

            this.useTunnel = settings.useTunnel;
            this.localPort = settings.localPort;

            this.useRemoteSave = settings.useRemoteSave;
            this.directory = settings.directory;
        }
    }

    /**
     * Gets a String representation of the ServerSettings.
     *
     * @return A String representation.
     */
    public String toString() {
        return "ServerSettings[news server=" + getNewsServer() +
               ", news port=" + getNewsPort() + ", use ssh=" + useSSH() +
               ", ssh server=" + getSSHServer() + ", ssh port=" +
               getSSHPort() + ", username=" + getUsername() +
               ", use ssh tunnel=" + useTunnel() + ", local port=" +
               getLocalPort() + ", use remote save=" + useRemoteSave() +
               ", directory=" + getDirectory() + "]";
    }
}
