/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.nntp;

import java.util.Date;
import java.util.Iterator;

import nan.exception.nntp.InvalidArticleException;
import nan.exception.nntp.NNTPConnectException;
import nan.exception.nntp.UnconnectedException;

import nan.model.Message;
import nan.model.MessageHeader;
import nan.model.Newsgroup;
import nan.model.NewsgroupList;

/**
 * An interface for NNTP client connections.
 *
 * @author John David Ratliff
 * @version 1.7, 12/28/03
 */
public interface NNTPClient {
    /**
     * Connects to the NNTP server.
     *
     * @param hostname The NNTP server's hostname.
     * @param port The NNTP server's port.
     *
     * @throws NNTPConnectException if the connection failed.
     */
    public void connect(String hostname, int port)
                 throws NNTPConnectException;

    /**
     * Disconnects from the server.
     */
    public void disconnect();

    /**
     * Checks if this client is connected.
     *
     * @return true if this client is connected; false otherwise.
     */
    public boolean isConnected();

    /**
     * Checks if posting is allowed on this server.
     *
     * @return true if posting is allowed; false otherwise.
     */
    public boolean isPostingAllowed();

    /**
     * Gets the list of available Newsgroups from the NNTP server.
     *
     * @return The NewsgroupList of available Newsgroups.
     *
     * @throws UnconnectedException if this client is not connected.
     */
    public NewsgroupList getNewsgroupList() throws UnconnectedException;

    /**
     * Selects the current Newsgroup.
     *
     * @param newsgroup The Newsgroup to select.
     *
     * @return true if the Newsgroup was selected; false otherwise.
     *
     * @throws UnconnectedException if this client is not connected.
     */
    public boolean selectNewsgroup(Newsgroup newsgroup)
                            throws UnconnectedException;

    /**
     * Gets the MessageHeader of a newsgroup article.
     *
     * @param article The article ID in the current group.
     *
     * @return The MessageHeader.
     *
     * @throws UnconnectedException if this client is not connected.
     * @throws InvalidArticleException if the article does not exist on this
     *         server.
     */
    public MessageHeader getHeader(int article)
                            throws UnconnectedException, 
                                   InvalidArticleException;

    /**
     * Gets the MessageHeader of a newsgroup article.
     *
     * @param messageId The message ID of the article.
     *
     * @return The MessageHeader.
     *
     * @throws UnconnectedException if this client is not connected.
     * @throws InvalidArticleException if the article does not exist on this
     *         server.
     */
    public MessageHeader getHeader(String messageId)
                            throws UnconnectedException, 
                                   InvalidArticleException;

    /**
     * Gets a message from the news server.
     *
     * @param article The article ID in the current group.
     *
     * @return The Message.
     *
     * @throws UnconnectedException if this client is not connected.
     * @throws InvalidArticleException if the article does not exist on this
     *         server.
     */
    public Message getMessage(int article)
                       throws UnconnectedException, InvalidArticleException;

    /**
     * Gets a message from the news server. If the MessageHeader argument is
     * null, the MessageHeader will be retrieved as well. If the
     * MessageHeader is not null, it will be added as the MessageHeader of
     * the Message.
     *
     * @param article The article ID in the current group.
     * @param header The MessageHeader of this article.
     *
     * @return The Message.
     *
     * @throws UnconnectedException if this client is not connected.
     * @throws InvalidArticleException if the article does not exist on this
     *         server.
     */
    public Message getMessage(int article, MessageHeader header)
                       throws UnconnectedException, InvalidArticleException;

    /**
     * Gets a message from the news server.
     *
     * @param messageId The message ID of the article.
     *
     * @return The Message.
     *
     * @throws UnconnectedException if this client is not connected.
     * @throws InvalidArticleException if the article does not exist on this
     *         server.
     */
    public Message getMessage(String messageId)
                       throws UnconnectedException, InvalidArticleException;

    /**
     * Gets a message from the news server. If the MessageHeader argument is
     * null, the MessageHeader will be retrieved as well. If the
     * MessageHeader is not null, it will be added as the MessageHeader of
     * the Message.
     *
     * @param messageId The message ID of the article.
     * @param header The MessageHeader of this article.
     *
     * @return The Message.
     *
     * @throws UnconnectedException if this client is not connected.
     * @throws InvalidArticleException if the article does not exist on this
     *         server.
     */
    public Message getMessage(String messageId, MessageHeader header)
                       throws UnconnectedException, InvalidArticleException;

    /**
     * Gets the list of Newsgroups added to the server since a Date.
     *
     * @param when The Date after which the newsgroups were added.
     *
     * @return The NewsgroupList of recent newsgroups, which may be empty.
     *
     * @throws UnconnectedException if this client is not connected.
     */
    public NewsgroupList getNewsgroupsSince(Date when)
                                     throws UnconnectedException;

    /**
     * Gets the message IDs in an iterator of all messages added to a
     * Newsgroup since a Date.
     *
     * @param newsgroup The Newsgroup to check for new messages in.
     * @param when The Date after which the messages were added.
     *
     * @return An Iterator of the message IDs of the new messages.
     *
     * @throws UnconnectedException if this client is not connected.
     */
    public Iterator getMessagesSince(Newsgroup newsgroup, Date when)
                              throws UnconnectedException;

    /**
     * Posts a message to the NNTP server.
     *
     * @param message The Message to post.
     *
     * @return true if the message was posted; false otherwise.
     *
     * @throws UnconnectedException if this client is not connected.
     */
    public boolean post(Message message) throws UnconnectedException;
}
