/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.nntp;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.StringReader;

import java.net.Socket;
import java.net.UnknownHostException;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import nan.NaN;

import nan.exception.nntp.InvalidArticleException;
import nan.exception.nntp.NNTPConnectException;
import nan.exception.nntp.NNTPException;
import nan.exception.nntp.NNTPResetException;
import nan.exception.nntp.UnconnectedException;

import nan.model.Attachment;
import nan.model.Message;
import nan.model.MessageHeader;
import nan.model.Newsgroup;
import nan.model.NewsgroupList;

import nan.tools.EventLogger;
import nan.tools.NNTPDateFormat;

import nan.view.WaitDialog;

/**
 * An implementation of the NNTP Client interface.
 *
 * @author John David Ratliff
 * @version 1.34, 1/6/04
 */
public class NNTPClientImpl implements NNTPClient {
    private Socket socket;
    protected BufferedReader reader;
    protected PrintWriter writer;
    private boolean connected;
    private boolean posting;
    private String hostname;
    private int port;
    private Newsgroup newsgroup;
    private WaitDialog dialog;
    private static final Pattern NEWSGROUP_PATTERN = Pattern.compile("^(.+) (\\d+) (\\d+) ([yYNnMm])$");
    private static final Pattern SELECT_PATTERN = Pattern.compile("^\\d+ (\\d+) (\\d+) .+$");
    private static final Pattern BEGIN_PATTERN = Pattern.compile("^begin \\d+ (.+)$",
                                                                 Pattern.MULTILINE);
    private static final Pattern END_PATTERN = Pattern.compile("^end$",
                                                               Pattern.MULTILINE);
    private static final String NNTP_DISCONNECT_COMMAND = "QUIT";
    private static final String NNTP_LIST_COMMAND = "LIST";
    private static final String NNTP_POST_COMMAND = "POST";
    private static final String NNTP_GROUP_COMMAND = "GROUP";
    private static final String NNTP_BODY_COMMAND = "BODY";
    private static final String NNTP_HEAD_COMMAND = "HEAD";
    private static final String NNTP_NEWNEWS_COMMAND = "NEWNEWS";
    private static final String NNTP_NEWGROUPS_COMMAND = "NEWGROUPS";

    /**
     * The status response for NNTP connections that allow posting.
     */
    public static final int NNTP_CONNECT_OK = 200;

    /**
     * The status response for NNTP connections that forbid posting.
     */
    public static final int NNTP_CONNECT_NO_POSTING = 201;

    /**
     * The status response for disconnecting from the NNTP server.
     */
    public static final int NNTP_DISCONNECT_OK = 205;

    /**
     * The status response for successful LIST command.
     */
    public static final int NNTP_LIST_OK = 215;

    /**
     * The status response for POST command.
     */
    public static final int NNTP_POST_CONTINUE = 340;

    /**
     * The status response for a successful POST command.
     */
    public static final int NNTP_POST_OK = 240;

    /**
     * The status response for a successful GROUP command.
     */
    public static final int NNTP_GROUP_OK = 211;

    /**
     * The status response for a successful HEAD command.
     */
    public static final int NNTP_HEAD_OK = 221;

    /**
     * The status response for a successful BODY command.
     */
    public static final int NNTP_BODY_OK = 222;

    /**
     * The status response for a successful NEWNEWS command.
     */
    public static final int NNTP_NEWNEWS_OK = 230;

    /**
     * The status response for a successful NEWGROUPS command.
     */
    public static final int NNTP_NEWGROUPS_OK = 231;

    /**
     * Creates a new NNTPClientImpl object.
     */
    public NNTPClientImpl() {
    }

    /**
     * Reads the next response from the NNTP server.
     *
     * @return The NNTPResponse.
     *
     * @throws NNTPResetException if the connection is reset.
     */
    private NNTPResponse readResponse() throws NNTPResetException {
        NNTPResponseImpl response = new NNTPResponseImpl(this);

        response.readResponse();

        return response;
    }

    /**
     * Sends a line of text to the NNTP server.
     *
     * @param line The line to send.
     */
    private void sendLine(String line) {
        if (this.isConnected()) {
            this.writer.print(line);
            this.writer.print('\r');
            this.writer.print('\n');
            this.writer.flush();
        }
    }

    /**
     * Connects to the NNTP server.
     *
     * @param hostname The NNTP server's hostname.
     * @param port The NNTP server's port.
     *
     * @throws NNTPConnectException if the connection failed.
     */
    public void connect(String hostname, int port)
                 throws NNTPConnectException {
        if (isConnected()) {
            this.disconnect();
        }

        // save the hostname and port
        this.hostname = hostname;
        this.port = port;

        try {
            this.socket = new Socket(hostname, port);
            this.reader = new BufferedReader(new InputStreamReader(this.socket.getInputStream()));
            this.writer = new PrintWriter(this.socket.getOutputStream());

            this.connected = true;

            NNTPResponse response;

            try {
                response = readResponse();
            } catch (NNTPResetException e) {
                throw new NNTPConnectException();
            }

            if (response.getStatusCode() == NNTP_CONNECT_OK) {
                this.posting = true;
            } else if (response.getStatusCode() == NNTP_CONNECT_NO_POSTING) {
                this.posting = false;
            } else {
                this.disconnect();
            }
        } catch (UnknownHostException e) {
            // invalid hostname
            throw new NNTPConnectException();
        } catch (IOException e) {
            EventLogger.getInstance().logError("IOException during NNTPClientImpl.connect(" +
                                               hostname + ", port)...");

            throw new NNTPConnectException();
        }
    }

    /**
     * Reconnects to the NNTP server. Generally only called when an
     * NNTPResetException is caught.
     */
    private void reconnect() {
        if (this.hostname != null) {
            this.disconnect();
            this.connect(this.hostname, this.port);
        }
    }

    /**
     * Checks if this client is connected.
     *
     * @return true if this client is connected; false otherwise.
     */
    public boolean isConnected() {
        return (this.connected && this.socket.isConnected());
    }

    /**
     * Disconnects from the server.
     */
    public void disconnect() {
        if (this.isConnected()) {
            this.sendLine(NNTP_DISCONNECT_COMMAND);

            this.connected = false;

            try {
                this.socket.close();
            } catch (IOException e) {
                EventLogger.getInstance().logError("IOExcpetion during " +
                                                   "NNTPClientImpl.disconnect()...");
            }
        }
    }

    /**
     * Checks if posting is allowed on this server.
     *
     * @return true if posting is allowed; false otherwise.
     */
    public boolean isPostingAllowed() {
        return this.posting;
    }

    /**
     * Parses a Newsgroup response line into a Newsgroup.
     *
     * @param line The response line.
     *
     * @return The Newsgroup.
     */
    private Newsgroup parseNewsgroup(String line) {
        Matcher match = NEWSGROUP_PATTERN.matcher(line);

        if (match.matches()) {
            String name = match.group(1);
            int lastArticle = Integer.parseInt(match.group(2));
            int firstArticle = Integer.parseInt(match.group(3));
            boolean postingAllowed = (match.group(4).equalsIgnoreCase("y"));

            return new Newsgroup(name, postingAllowed,
                                 Newsgroup.DEFAULT_MESSAGES);
        } else {
            EventLogger.getInstance().logError("Matcher doesn't match on " +
                                               line +
                                               " in NNTPClientImpl.getNewsgroupList()...");
        }

        return null;
    }

    /**
     * Gets the list of available Newsgroups from the NNTP server.
     *
     * @return The NewsgroupList of available Newsgroups.
     *
     * @throws UnconnectedException if this client is not connected.
     * @throws NNTPException if an unknown NNTP problem occurs.
     */
    public NewsgroupList getNewsgroupList() throws UnconnectedException {
        if (!this.isConnected()) {
            throw new UnconnectedException();
        }

        NewsgroupList list = new NewsgroupList();

        NNTPResponse response = null;

        try {
            // send the list command to the server
            this.sendLine(NNTP_LIST_COMMAND);

            // read the response from the server
            response = this.readResponse();
        } catch (NNTPResetException e) {
            // connection was reset
            this.reconnect();

            EventLogger.getInstance().logMessage("NNTP connection was reset during " +
                                                 "NNTPImpl.getNewsgroupList()...");

            throw e;
        }

        if (response.getStatusCode() != NNTP_LIST_OK) {
            throw new NNTPException();
        }

        try {
            BufferedReader reader = new BufferedReader(new StringReader(response.getText()));
            String line;

            int count = 0;
            int total = response.getLineCount();

            while ((line = reader.readLine()) != null) {
                // do status messages if we have a dialog
                if (dialog != null) {
                    String message = "Reading Newsgroup " + count + " of " +
                                     total + ".";
                    dialog.setText(message);
                }

                Newsgroup newsgroup = parseNewsgroup(line);

                if (newsgroup != null) {
                    list.add(newsgroup);
                }

                count++;
            }
        } catch (IOException e) {
            // this can NEVER happen...
            EventLogger.getInstance().logError("IOException during " +
                                               "NNTPClientImpl.getNewsgroupList()...");
        }

        // sort the list before returning it
        list.sort();

        // take the dialog down if it is up
        if (dialog != null) {
            while (!dialog.isVisible())
                ;

            dialog.setVisible(false);
        }

        return list;
    }

    /**
     * Selects the current Newsgroup.
     *
     * @param newsgroup The Newsgroup to select.
     *
     * @return true if the Newsgroup was selected; false otherwise.
     *
     * @throws UnconnectedException if this client is not connected.
     */
    public boolean selectNewsgroup(Newsgroup newsgroup)
                            throws UnconnectedException {
        if (!isConnected()) {
            throw new UnconnectedException();
        }

        String command = NNTP_GROUP_COMMAND + " " + newsgroup.getName();

        NNTPResponse response;

        try {
            this.sendLine(command);

            response = this.readResponse();
        } catch (NNTPResetException e) {
            // connection reset
            this.reconnect();

            EventLogger.getInstance().logMessage("NNTP connection was reset during " +
                                                 "NNTPImpl.selectNewsgroup()...");

            return false;
        }

        if (response.getStatusCode() != NNTP_GROUP_OK) {
            return false;
        }

        this.newsgroup = newsgroup;

        Matcher match = SELECT_PATTERN.matcher(response.getStatusMessage());

        if (match.matches()) {
            int first = Integer.parseInt(match.group(1));
            int last = Integer.parseInt(match.group(2));

            newsgroup.setFirstArticle(first);
            newsgroup.setLastArticle(last);
        } else {
            EventLogger.getInstance().logError("Pattern doesn't match " +
                                               response.getStatusMessage() +
                                               " in NNTPClientImpl.selectNewsgroup()...");

            return false;
        }

        return true;
    }

    /**
     * Gets the list of Newsgroups added to the server since a Date.
     *
     * @param when The Date after which the newsgroups were added.
     *
     * @return The NewsgroupList of recent newsgroups, which may be empty.
     *
     * @throws UnconnectedException if this client is not connected.
     * @throws NNTPException if an unknown NNTP problem occurs.
     */
    public NewsgroupList getNewsgroupsSince(Date when)
                                     throws UnconnectedException {
        if (!isConnected()) {
            throw new UnconnectedException();
        }

        NewsgroupList list = new NewsgroupList();
        NNTPResponse response;

        try {
            sendLine(NNTP_NEWGROUPS_COMMAND + " " +
                     NNTPDateFormat.toSinceDateString(when));
            response = this.readResponse();
        } catch (NNTPResetException e) {
            // connection was reset...
            this.reconnect();

            EventLogger.getInstance().logMessage("NNTP connection was reset during " +
                                                 "NNTPImpl.getNewsgroupsSince()...");

            throw e;
        }

        if (response.getStatusCode() != NNTP_NEWGROUPS_OK) {
            throw new NNTPException();
        }

        try {
            BufferedReader reader = new BufferedReader(new StringReader(response.getText()));
            String line;

            while ((line = reader.readLine()) != null) {
                Newsgroup newsgroup = parseNewsgroup(line);

                if (newsgroup != null) {
                    list.add(newsgroup);
                }
            }
        } catch (IOException e) {
            // this seems unlikely to occur...
            EventLogger.getInstance().logError("IOException during " +
                                               "NNTPClientImpl.getNewsgroupsSince()...");
        }

        return list;
    }

    /**
     * Gets the message IDs in an iterator of all messages added to a
     * Newsgroup since a Date.
     *
     * @param newsgroup The Newsgroup to check for new messages in.
     * @param when The Date after which the messages were added.
     *
     * @return An Iterator of the message IDs of the new messages.
     *
     * @throws UnconnectedException if this client is not connected.
     * @throws NNTPException if an unknown NNTP problem occurs.
     */
    public Iterator getMessagesSince(Newsgroup newsgroup, Date when)
                              throws UnconnectedException {
        if (!isConnected()) {
            throw new UnconnectedException();
        }

        List list = new ArrayList();
        NNTPResponse response;

        try {
            this.sendLine(NNTP_NEWNEWS_COMMAND + " " + newsgroup.getName() +
                          " " + NNTPDateFormat.toSinceDateString(when));
            response = this.readResponse();
        } catch (NNTPResetException e) {
            // connection was reset...
            this.reconnect();

            EventLogger.getInstance().logMessage("NNTP connection was reset during " +
                                                 "NNTPImpl.getMessagesSince()...");

            throw e;
        }

        if (response.getStatusCode() != NNTP_NEWNEWS_OK) {
            throw new NNTPException();
        }

        try {
            BufferedReader reader = new BufferedReader(new StringReader(response.getText()));
            String line;

            while ((line = reader.readLine()) != null) {
                list.add(line);
            }
        } catch (IOException e) {
            // this seems unlikely to occur
            EventLogger.getInstance().logError("IOException during " +
                                               "NNTPClientImpl.getMessagesSince()...");
        }

        return list.iterator();
    }

    /**
     * Parses and returns a MessageHeader from the server.
     *
     * @param command The command to retrieve the message header.
     *
     * @return The MessageHeader.
     *
     * @throws InvalidArticleException if the article does not exist on this
     *         server.
     */
    private MessageHeader parseHeader(String command)
                               throws InvalidArticleException {
        MessageHeader header = new MessageHeader();

        NNTPResponse response;

        try {
            this.sendLine(command);

            response = this.readResponse();
        } catch (NNTPResetException e) {
            // connection reset
            this.reconnect();

            EventLogger.getInstance().logMessage("NNTP connection was reset during " +
                                                 "NNTPImpl.parseHeader()...");

            throw e;
        }

        if (response.getStatusCode() != NNTP_HEAD_OK) {
            throw new InvalidArticleException();
        }

        try {
            BufferedReader reader = new BufferedReader(new StringReader(response.getText()));
            String line;
            String lastHeader = null;

            while ((line = reader.readLine()) != null) {
                int index = line.indexOf(':');

                if (index == -1) {
                    // continuation line
                    String value = header.getValue(lastHeader);
                    value += line;

                    header.replaceHeader(lastHeader, value);
                } else {
                    // new header
                    String name = line.substring(0, index);
                    String value = line.substring(index + 2);

                    header.addHeader(name, value);
                    lastHeader = name;
                }
            }
        } catch (IOException e) {
            // I don't really believe this will EVER happen...
            EventLogger.getInstance().logError("IOException during " +
                                               "NNTPClientImpl.parseMessage()...");
        }

        return header;
    }

    /**
     * Gets the MessageHeader of a newsgroup article.
     *
     * @param article The article ID in the current group.
     *
     * @return The MessageHeader.
     *
     * @throws UnconnectedException if this client is not connected.
     * @throws InvalidArticleException if the article does not exist on this
     *         server.
     */
    public MessageHeader getHeader(int article)
                            throws UnconnectedException, 
                                   InvalidArticleException {
        if (!isConnected()) {
            throw new UnconnectedException();
        }

        MessageHeader header = parseHeader(NNTP_HEAD_COMMAND + " " +
                                           article);
        header.addHeader("X-Article-Number", "" + article);

        return header;
    }

    /**
     * Gets the MessageHeader of a newsgroup article.
     *
     * @param messageId The message ID of the article.
     *
     * @return The MessageHeader.
     *
     * @throws UnconnectedException if this client is not connected.
     * @throws InvalidArticleException if the article does not exist on this
     *         server.
     */
    public MessageHeader getHeader(String messageId)
                            throws UnconnectedException, 
                                   InvalidArticleException {
        if (!isConnected()) {
            throw new UnconnectedException();
        }

        MessageHeader header = parseHeader(NNTP_HEAD_COMMAND + " " +
                                           messageId);

        // add X-Article-Number property
        String xref = header.getValue("xref");
        String article = null;

        while (article == null) {
            if (xref.startsWith(newsgroup.getName())) {
                int colon = xref.indexOf(':');
                int space = xref.indexOf(' ', colon);

                if (space != -1) {
                    article = xref.substring(colon + 1, space);
                } else {
                    article = xref.substring(colon + 1);
                }
            } else {
                int index = xref.indexOf(' ');
                xref = xref.substring(index + 1);
            }
        }

        header.addHeader("X-Article-Number", article);

        return header;
    }

    /**
     * Gets the body of a message on the server.
     *
     * @param command The command to retrieve the message.
     *
     * @return The body of the message.
     *
     * @throws InvalidArticleException if the article does not exist on this
     *         server.
     */
    private String getBody(String command) throws InvalidArticleException {
        NNTPResponse response;

        try {
            this.sendLine(command);

            response = this.readResponse();
        } catch (NNTPResetException e) {
            // connection reset
            this.reconnect();

            EventLogger.getInstance().logMessage("NNTP connection was reset during " +
                                                 "NNTPImpl.getBody()...");

            throw e;
        }

        if (response.getStatusCode() != NNTP_BODY_OK) {
            throw new InvalidArticleException();
        }

        return response.getText();
    }

    /**
     * Parses attachments from the Message.
     *
     * @param message The Message to parse.
     */
    private void parseAttachments(Message message) {
        String body = message.getBody();
        boolean done = false;

        do {
            Matcher first = BEGIN_PATTERN.matcher(body);
            Matcher last = END_PATTERN.matcher(body);

            if (first.find() && last.find(first.end())) {
                // we found us an attachment...
                int alpha = first.end() + 1;
                int omega = last.start() - 1;

                String name = first.group(1);
                String data = body.substring(alpha, omega);

                Attachment attachment = new Attachment(name, data);
                message.addAttachment(attachment);

                // cut it out of the body
                int gamma = first.start();
                int theta = last.end();

                body = body.substring(0, gamma) + body.substring(theta);
            } else {
                done = true;
            }
        } while (!done);

        message.setBody(body);
    }

    /**
     * Gets a message from the news server. If the MessageHeader argument is
     * null, the MessageHeader will be retrieved as well. If the
     * MessageHeader is not null, it will be added as the MessageHeader of
     * the Message.
     *
     * @param article The article ID in the current group.
     * @param header The MessageHeader of this article.
     *
     * @return The Message.
     *
     * @throws UnconnectedException if this client is not connected.
     * @throws InvalidArticleException if the article does not exist on this
     *         server.
     */
    public Message getMessage(int article, MessageHeader header)
                       throws UnconnectedException, InvalidArticleException {
        if (!isConnected()) {
            throw new UnconnectedException();
        }

        if (header == null) {
            header = getHeader(article);
        }

        String body = getBody(NNTP_BODY_COMMAND + " " + article);

        Message message = new Message(header, body);
        parseAttachments(message);

        return message;
    }

    /**
     * Gets a message from the news server.
     *
     * @param article The article ID in the current group.
     *
     * @return The Message.
     *
     * @throws UnconnectedException if this client is not connected.
     * @throws InvalidArticleException if the article does not exist on this
     *         server.
     */
    public Message getMessage(int article)
                       throws UnconnectedException, InvalidArticleException {
        return getMessage(article, null);
    }

    /**
     * Gets a message from the news server. If the MessageHeader argument is
     * null, the MessageHeader will be retrieved as well. If the
     * MessageHeader is not null, it will be added as the MessageHeader of
     * the Message.
     *
     * @param messageId The message ID of the article.
     * @param header The MessageHeader of this article.
     *
     * @return The Message.
     *
     * @throws UnconnectedException if this client is not connected.
     * @throws InvalidArticleException if the article does not exist on this
     *         server.
     */
    public Message getMessage(String messageId, MessageHeader header)
                       throws UnconnectedException, InvalidArticleException {
        if (!isConnected()) {
            throw new UnconnectedException();
        }

        if (header == null) {
            header = getHeader(messageId);
        }

        String body = getBody(NNTP_BODY_COMMAND + " " + messageId);

        Message message = new Message(header, body);
        parseAttachments(message);

        return message;
    }

    /**
     * Gets a message from the news server.
     *
     * @param messageId The message ID of the article.
     *
     * @return The Message.
     *
     * @throws UnconnectedException if this client is not connected.
     * @throws InvalidArticleException if the article does not exist on this
     *         server.
     */
    public Message getMessage(String messageId)
                       throws UnconnectedException, InvalidArticleException {
        return getMessage(messageId, null);
    }

    /**
     * Posts a message to the NNTP server.
     *
     * @param message The Message to post.
     *
     * @return true if the message was posted; false otherwise.
     *
     * @throws UnconnectedException if this client is not connected.
     */
    public boolean post(Message message) throws UnconnectedException {
        if (!isPostingAllowed()) {
            return false;
        }

        if (!isConnected()) {
            throw new UnconnectedException();
        }

        NNTPResponse response;

        try {
            // send the POST command
            this.sendLine(NNTP_POST_COMMAND);

            response = this.readResponse();
        } catch (NNTPResetException e) {
            // connection was reset
            this.reconnect();

            EventLogger.getInstance().logMessage("NNTP connection was reset during " +
                                                 "NNTPImpl.post()...");

            throw e;
        }

        // make sure we get the green light
        if (response.getStatusCode() != NNTP_POST_CONTINUE) {
            return false;
        }

        try {
            String line;
            BufferedReader reader;

            reader = new BufferedReader(new StringReader(message.getHeader()
                                                                .getHeaders()));

            // send all the headers
            while ((line = reader.readLine()) != null) {
                this.sendLine(line);
            }

            reader.close();

            // separate the headers from the body
            this.sendLine("");

            reader = new BufferedReader(new StringReader(message.getBody()));

            // send the body
            while ((line = reader.readLine()) != null) {
                if ((line.length() > 0) && (line.charAt(0) == '.')) {
                    // must double first periods
                    line = "." + line;
                }

                this.sendLine(line);
            }

            reader.close();

            // send the attachments
            if (message.getAttachmentCount() > 0) {
                this.sendLine("");

                for (int i = 0; i < message.getAttachmentCount(); i++) {
                    Attachment attachment = message.getAttachment(i);

                    reader = new BufferedReader(new StringReader(attachment.getUUEData()));

                    while ((line = reader.readLine()) != null) {
                        this.sendLine(line);
                    }
                }
            }

            // send the body terminator
            this.sendLine(".");
        } catch (IOException e) {
            // this should NEVER happen...
            // make sure we've terminated...
            this.sendLine(".");

            EventLogger.getInstance().logError("IOException during NNTPCliemtImpl.post()...");
        }

        // read the server's response to the message
        response = this.readResponse();

        if (response.getStatusCode() != NNTP_POST_OK) {
            return false;
        }

        return true;
    }

    /**
     * Sets the wait dialog that is waiting on a task to finish.
     *
     * @param dialog The WaitDialog.
     */
    public void setWaitDialog(WaitDialog dialog) {
        this.dialog = dialog;
    }

    /**
     * Returns a String representation of this NNTPClient.
     *
     * @return A String representation.
     */
    public String toString() {
        return "NNTPClientImpl[socket=" + this.socket + ", connected=" +
               isConnected() + "]";
    }

    /**
     * A method to test this class for correctness.
     */
    public static void test() {
        System.out.println("start of test cases for NNTPClientImpl...");

        Date when = new Date();
        NNTPClient client = new NNTPClientImpl();

        try {
            System.out.println("connecting to an invalid server...");
            client.connect("unknown.host", 119);

            assert false;
        } catch (NNTPConnectException e) {
            System.out.println("Caught NNTPConnectException as expected...");
        } catch (Exception e) {
            e.printStackTrace();

            assert false;
        }

        try {
            assert (!client.isConnected());

            // THIS TEST MIGHT BE A FAILURE ON MOST MACHINES. YOU'LL
            // NEED THE NAME AND PORT OF A VALID NNTP SERVER TO CONNECT TO
            System.out.println("connecting to a valid NNTP server...");

            // connect(String, int)
            client.connect("localhost", 119);

            // isConnected()
            assert (client.isConnected());

            System.out.println("reading the list of newsgroups from the server...");

            // getNewsgroupList()
            NewsgroupList list = client.getNewsgroupList();

            if (list.getSize() > 0) {
                Newsgroup group = list.get(list.getSize() - 1);

                System.out.println("first group = " + list.get(0));
                System.out.println("last group = " + group);

                // isPostingAllowed()
                if (client.isPostingAllowed() && group.isPostingAllowed()) {
                    // test message
                    MessageHeader header = new MessageHeader();
                    header.addHeader("From",
                                     NaN.APP_NAME + " " + NaN.APP_VERSION);
                    header.addHeader("Subject",
                                     "Test Message at " + new Date());
                    header.addHeader("Newsgroups", group.getName());

                    Message message = new Message(header,
                                                  "Test Post in NNTPClientImpl.test()...");

                    // post(Message)
                    if (!client.post(message)) {
                        assert false;
                    }

                    // getMessagesSince(Newsgroup, Date)
                    System.out.println("testing getMessagesSince()...");

                    Iterator iterator = client.getMessagesSince(group, when);

                    while (iterator.hasNext()) {
                        String messageId = (String)iterator.next();
                        System.out.println("new message id in group " +
                                           group.getName() + " = " +
                                           messageId);
                    }
                }

                // getNewsgroupsSince(Date)
                System.out.println("testing getNewsgroupsSince()...");

                // 7 days ago (7 days * 24 hr/day * 60 min/hr * 60 sec/min * 1000 ms/sec)
                when = new Date(new Date().getTime() -
                                (7 * 24 * 60 * 60 * 1000));

                list = client.getNewsgroupsSince(when);

                for (int i = 0; i < list.getSize(); i++) {
                    Newsgroup newsgroup = list.get(i);
                    System.out.println("new newsgroup = " +
                                       newsgroup.getName());
                }

                // selectNewsgroup(Newsgroup)
                System.out.println("Selecting newsgroup " + group);
                assert (client.selectNewsgroup(group));

                if (group.getLastArticle() > group.getFirstArticle()) {
                    // theoretically, this group has articles in it...
                    // no way to be sure though...
                    MessageHeader header = null;
                    Message message = null;

                    // getHeader(int)
                    System.out.println("retrieving the header of message " +
                                       group.getFirstArticle() +
                                       " from group " + group.getName());

                    try {
                        header = client.getHeader(group.getFirstArticle());
                    } catch (InvalidArticleException e) {
                        assert false;
                    }

                    // getMessage(String, MessageHeader)
                    String messageId = header.getValue("message-id");
                    System.out.println("retrieving the body of message with id '" +
                                       messageId + "' from group " +
                                       group.getName());

                    try {
                        message = client.getMessage(messageId, header);
                    } catch (InvalidArticleException e) {
                        assert false;
                    }

                    System.out.println("Message Headers:\n" +
                                       header.getHeaders());
                    System.out.println("Message Body:\n" +
                                       message.getBody());

                    // getHeader(String)
                    System.out.println("testing getHeader(String)...");

                    try {
                        header = client.getHeader(messageId);

                        assert (header != null);
                    } catch (InvalidArticleException e) {
                        assert false;
                    }

                    // getMessage(int)
                    System.out.println("testing getMessage(int)...");

                    try {
                        message = client.getMessage(group.getFirstArticle());

                        assert (message != null);
                        assert (message.getBody() != null);
                        assert (message.getHeader() != null);
                    } catch (InvalidArticleException e) {
                        assert false;
                    }

                    // getMessage(String)
                    System.out.println("testing getMessage(String)...");

                    try {
                        message = client.getMessage(messageId);

                        assert (message != null);
                        assert (message.getBody() != null);
                        assert (message.getHeader() != null);
                    } catch (InvalidArticleException e) {
                        assert false;
                    }

                    // getMessage(int, MessageHeader)
                    System.out.println("testing getMessage(int, MessageHeader)...");

                    try {
                        message = client.getMessage(group.getFirstArticle(),
                                                    header);

                        assert (message != null);
                        assert (message.getBody() != null);
                        assert (message.getHeader() != null);
                    } catch (InvalidArticleException e) {
                        assert false;
                    }

                    System.out.println("testing invalid getHeader(int)...");

                    int article = group.getLastArticle() + 20;

                    try {
                        client.getHeader(article);

                        assert false;
                    } catch (InvalidArticleException e) {
                        System.out.println("caught InvalidArticleException as expected...");
                    }

                    System.out.println("testing invalid getMessage(int)...");

                    try {
                        client.getMessage(article);

                        assert false;
                    } catch (InvalidArticleException e) {
                        System.out.println("caught InvalidArticleException as expected...");
                    }

                    messageId = "<stupid.messageId.that.better.not.be.valid>";

                    System.out.println("testing invalid getHeader(String)...");

                    try {
                        client.getHeader(messageId);

                        assert false;
                    } catch (InvalidArticleException e) {
                        System.out.println("caught InvalidArticleException as expected...");
                    }

                    System.out.println("testing invalid getMessage(String)...");

                    try {
                        client.getMessage(messageId);

                        assert false;
                    } catch (InvalidArticleException e) {
                        System.out.println("caught InvalidArticleException as expected...");
                    }
                }
            }

            Newsgroup group = new Newsgroup("invalid.name.xxx.wontbehere.ihope",
                                            false, Newsgroup.ALL_MESSAGES);
            System.out.println("selecting (hopefully) invalid newsgroup " +
                               group);
            assert (!client.selectNewsgroup(group));

            // disconnect()
            System.out.println("disconnecting from the server...");
            client.disconnect();

            assert (!client.isConnected());
        } catch (Exception e) {
            e.printStackTrace();

            assert false;
        }

        System.out.println("end of test cases for NNTPClientImpl...");
    }

    /**
     * An implementation of the NNTPResponse interface.
     *
     * @author John David Ratliff
     * @version 1.11, 12/28/03
     */
    private static class NNTPResponseImpl implements NNTPResponse {
        private NNTPClientImpl client;
        private String text;
        private int lineCount;
        private int statusCode;
        private String statusMessage;
        private boolean readText;
        private static final Pattern RESPONSE = Pattern.compile("^(\\d\\d\\d) (.+)$");

        /**
         * Creates a new NNTPResponseImpl object.
         *
         * @param client The NNTPClient.
         */
        protected NNTPResponseImpl(NNTPClientImpl client) {
            this.client = client;
        }

        /**
         * Parses the status line into status code and status message
         * components.
         *
         * @param line The line to parse.
         */
        private void parseStatusLine(String line) {
            Matcher match = RESPONSE.matcher(line);

            if (match.matches()) {
                this.statusCode = Integer.parseInt(match.group(1));
                this.statusMessage = match.group(2);
            } else {
                EventLogger.getInstance().logError("Matcher doesn't match " +
                                                   line +
                                                   " in NNTPResponseImpl.processStatusLine()...");
            }
        }

        /**
         * Reads the next response from the server.
         *
         * @throws NNTPResetException if the connection is reset.
         */
        public void readResponse() throws NNTPResetException {
            String line;

            try {
                // read the status line
                line = this.client.reader.readLine();

                if (line == null) {
                    throw new NNTPResetException();
                }

                // parse the status line
                parseStatusLine(line);
            } catch (IOException e) {
                // this will hopefully not happen too often...
                EventLogger.getInstance().logError("IOException during " +
                                                   "NNTPClientImpl.readResponse()...");
            }
        }

        /**
         * Reads the text response from the server. This should be called
         * ONLY after readResponse()!
         *
         * @throws NNTPResetException if the connection is reset during the
         *         read.
         */
        private void readText() throws NNTPResetException {
            // read text response
            StringBuffer buffer = new StringBuffer();
            String line;

            try {
                boolean done = false;

                while (!done) {
                    line = this.client.reader.readLine();

                    if (line == null) {
                        EventLogger.getInstance().logWarning("Line is null during " +
                                                             "NNTPResponseImpl.readResponse()...");

                        throw new NNTPResetException();
                    } else if ((line.length() == 1) &&
                               (line.charAt(0) == '.')) {
                        done = true;
                    } else if ((line.length() > 0) &&
                               (line.charAt(0) == '.')) {
                        // catch doubled first period
                        line = line.substring(1);
                    }

                    if (!done && (line != null)) {
                        buffer.append(line);
                        buffer.append('\n');

                        // update the line count
                        this.lineCount++;
                    }
                }

                this.text = buffer.toString();
            } catch (IOException e) {
                // this will hopefully not happen too often...
                EventLogger.getInstance().logError("IOException during " +
                                                   "NNTPClientImpl.readText()...");
            }
        }

        /**
         * Gets the number of lines in the text component.
         *
         * @return The line count.
         */
        public int getLineCount() {
            return this.lineCount;
        }

        /**
         * Asks if this response has a text component.
         *
         * @return true if this response has text; false otherwise.
         */
        public boolean hasText() {
            return (this.lineCount > 0);
        }

        /**
         * Gets the text of this response.
         *
         * @return The text.
         */
        public String getText() {
            if (!this.readText) {
                this.readText();
            }

            return this.text;
        }

        /**
         * Gets the status code of the NNTP response.
         *
         * @return The status code.
         */
        public int getStatusCode() {
            return this.statusCode;
        }

        /**
         * Gets the status message.
         *
         * @return The status message.
         */
        public String getStatusMessage() {
            return this.statusMessage;
        }

        /**
         * Returns a String representation of this NNTPResponse.
         *
         * @return A String representation.
         */
        public String toString() {
            return "NNTPClientImpl[text=" + getText() + ", line count=" +
                   getLineCount() + ", status code=" + getStatusCode() +
                   ", status message=" + getStatusMessage() + "]";
        }
    }
}
