/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.ssh;

import java.io.File;
import java.io.IOException;

import java.net.Socket;
import java.net.UnknownHostException;

import java.util.Properties;

import com.mindbright.jca.security.SecureRandom;

import com.mindbright.ssh2.SSH2Exception;
import com.mindbright.ssh2.SSH2Preferences;
import com.mindbright.ssh2.SSH2SFTPTransfer;
import com.mindbright.ssh2.SSH2SimpleClient;
import com.mindbright.ssh2.SSH2Transport;

import com.mindbright.util.RandomSeed;
import com.mindbright.util.SecureRandomAndPad;

import nan.NaN;

import nan.exception.ssh.SSHConnectException;

import nan.model.ServerSettings;

import nan.tools.EventLogger;

/**
 * A class to perform SSH services.
 *
 * @author John David Ratliff
 * @version 1.6, 1/4/04
 */
public class SSHConnection {
    private RandomSeed seed;
    private Properties properties;
    private SSH2Preferences preferences;
    private SecureRandomAndPad random;
    private SSH2Transport transport;
    private SSH2SimpleClient client;
    private String hostname;
    private int port;
    private boolean connected;
    private SSHTunnel tunnel;
    private static final SSHConnection SINGLETON = new SSHConnection();

    /**
     * Creates a new SSHConnection object.
     */
    private SSHConnection() {
        // I am a singleton.
        // do not allow anyone else to create one of me...
        seed = new RandomSeed();

        properties = new Properties();
        properties.setProperty("auth-method", "password");

        preferences = new SSH2Preferences(properties);
        random = new SecureRandomAndPad(new SecureRandom(seed.getBytesBlocking(20,
                                                                               false)));
    }

    /**
     * Returns the singleton instance of SSHConnection.
     *
     * @return The only SSHConnection.
     */
    public static SSHConnection getInstance() {
        return SINGLETON;
    }

    /**
     * Asks if we are connected to the SSH server.
     *
     * @return true if the connection is open; false otherwise.
     */
    public boolean isConnected() {
        return (connected && (transport != null) &&
               (transport.isConnected()));
    }

    /**
     * Asks if we are connected to a particular server on a particular port.
     *
     * @param hostname The hostname.
     * @param port The port.
     *
     * @return true if an open connection exists to this server:port; false
     *         otherwise.
     */
    public boolean isConnectedTo(String hostname, int port) {
        return (isConnected() && (hostname.equals(this.hostname)) &&
               (port == this.port));
    }

    /**
     * Connects to the SSH server.
     *
     * @param settings The Settings used to connect.
     *
     * @throws SSHConnectException if the connection fails to establish.
     */
    public void connect(ServerSettings settings) throws SSHConnectException {
        if (isConnected()) {
            close();
        }

        this.hostname = settings.getSSHServer();
        this.port = settings.getSSHPort();

        String username = settings.getUsername();
        String password = settings.getPassword();

        // setup the SSH properties...
        properties.setProperty("server", hostname);
        properties.setProperty("port", "" + port);
        properties.setProperty("username", username);
        properties.setProperty("password", password);

        try {
            transport = new SSH2Transport(new Socket(hostname, port),
                                          preferences, random);
        } catch (UnknownHostException e) {
            EventLogger.getInstance().logError("UnknownHostException during " +
                                               "SSHConnection.connect()...");

            throw new SSHConnectException();
        } catch (IOException e) {
            EventLogger.getInstance().logError("IOException during SSHConnection.connect()...");

            throw new SSHConnectException();
        }

        try {
            client = new SSH2SimpleClient(transport, username, password);
            connected = true;
        } catch (SSH2Exception e) {
            transport.normalDisconnect("Goodbye");

            EventLogger.getInstance().logError("SSH2Exception during SSHConnection.connect()...");

            throw new SSHConnectException();
        }
    }

    /**
     * Close the SSH connection.
     */
    public void close() {
        if (isConnected()) {
            if (tunnel != null) {
                tunnel.close();
            }

            transport.normalDisconnect("Goodbye");
        }
    }

    /**
     * Opens an SSH tunnel on this connection.
     *
     * @param settings The Settings to use.
     *
     * @return true if the tunnel was opened; false otherwise.
     */
    public boolean openTunnel(ServerSettings settings) {
        // only allow one tunnel at a time...
        if (tunnel != null) {
            tunnel.close();
        }

        String localHostname = "localhost";
        int localPort = settings.getLocalPort();

        String remoteHostname = settings.getNewsServer();
        int remotePort = settings.getNewsPort();

        tunnel = new SSHTunnel(client.getConnection(), localHostname,
                               localPort, remoteHostname, remotePort);
        tunnel.open();

        return tunnel.isOpen();
    }

    /**
     * Uploads a file to the SSH server.
     *
     * @param directory The server's directory.
     * @param filename The filename to upload.
     */
    public void upload(String directory, String filename) {
        if (isConnected()) {
            try {
                String[] names = new String[1];
                names[0] = filename;

                SSH2SFTPTransfer transfer = new SSH2SFTPTransfer(new File(NaN.BASE_DIRECTORY),
                                                                 this.client.getConnection());
                transfer.copyToRemote(names, directory, false);
            } catch (SSH2Exception e) {
                // this will be hopefully rare...
                EventLogger.getInstance().logError("SSH2Exception in SSHConnection.upload()...");
            } catch (IOException e) {
                // please don't happen...
                EventLogger.getInstance().logError("IOException in SSHConnection.upload()...");
            }
        }
    }

    /**
     * Downloads a file from the SSH server.
     *
     * @param directory The server's directory.
     * @param filename The filename to download.
     */
    public void download(String directory, String filename) {
        if (isConnected()) {
            try {
                String[] names = new String[1];
                names[0] = directory + (directory.endsWith("/") ? "" : "/") +
                           filename;

                SSH2SFTPTransfer transfer = new SSH2SFTPTransfer(new File(NaN.BASE_DIRECTORY),
                                                                 client.getConnection());
                transfer.copyToLocal(NaN.BASE_DIRECTORY, names, false);
            } catch (SSH2Exception e) {
                // this will be hopefully rare...
                EventLogger.getInstance().logError("SSH2Exception in SSHConnection.download()...");
            } catch (IOException e) {
                // please don't happen...
                EventLogger.getInstance().logError("IOException in SSHConnection.download()...");
                e.printStackTrace();
            }
        }
    }

    /**
     * Returns a String representation of this object.
     *
     * @return A String representation.
     */
    public String toString() {
        return "SSHConnection[seed=" + seed + ", properties=" + properties +
               ", preferences=" + preferences + ", random=" + random +
               ", transport=" + transport + ", client=" + client +
               ", connected=" + isConnected() + "]";
    }
}
