/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.ssh;

import java.io.IOException;

import com.mindbright.ssh2.SSH2Connection;

import nan.tools.EventLogger;

/**
 * A class for opening SSH tunnels.
 *
 * @author John David Ratliff
 * @version 1.2, 12/29/03
 */
class SSHTunnel {
    private SSH2Connection ssh;
    private String localHostname;
    private int localPort;
    private String remoteHostname;
    private int remotePort;
    private boolean open;

    /**
     * Creates a new SSHTunnel object.
     *
     * @param ssh The SSH2Connection to open the tunnels on.
     * @param localHostname The local hostname.
     * @param localPort The local port.
     * @param remoteHostname The remote hostname.
     * @param remotePort The remote port.
     */
    public SSHTunnel(SSH2Connection ssh, String localHostname,
                     int localPort, String remoteHostname, int remotePort) {
        this.ssh = ssh;
        this.localHostname = localHostname;
        this.localPort = localPort;
        this.remoteHostname = remoteHostname;
        this.remotePort = remotePort;
    }

    /**
     * Asks if this tunnel is open.
     *
     * @return true if the tunnel is open; false otherwise.
     */
    public boolean isOpen() {
        return this.open;
    }

    /**
     * Opens this tunnel.
     */
    public void open() {
        try {
            ssh.newLocalForward(localHostname, localPort, remoteHostname,
                                remotePort);
            open = true;
        } catch (IOException e) {
            // unable to create tunnel
            EventLogger.getInstance().logError("IOException opening tunnel from " +
                                               localHostname + ":" +
                                               localPort + " to " +
                                               remoteHostname + ":" +
                                               remotePort +
                                               " in SSHTunnel.open()...");
        }
    }

    /**
     * Closes this tunnel. Does not close connections already in progress
     * though, just prevents further connections through the tunnel.
     */
    public void close() {
        if (isOpen()) {
            ssh.deleteLocalForward(localHostname, localPort);
        }
    }

    /**
     * Returns a String representation of this object.
     *
     * @return A String representation.
     */
    public String toString() {
        return "SSHTunnel[ssh=" + ssh + ", local hostname=" + localHostname +
               ", local port=" + localPort + ", remote hostname=" +
               remoteHostname + ", remote port=" + remotePort + ", open=" +
               isOpen() + "]";
    }
}
