/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.tools;

import java.util.Enumeration;
import java.util.StringTokenizer;

import javax.swing.tree.DefaultMutableTreeNode;

import nan.model.Message;
import nan.model.MessageHeader;
import nan.model.MessageTree;
import nan.model.Newsgroup;

import nan.nntp.NNTPClient;

import nan.view.View;
import nan.view.WaitDialog;

/**
 * Downloads a message cache for offline reading.
 *
 * @author John David Ratliff
 * @version 1.1, 1/3/04
 */
public class BackgroundCacheDownloader extends AbstractBackgroundTask {
    private View waiter;
    private Newsgroup newsgroup;
    private NNTPClient client;
    private WaitDialog dialog;

    /**
     * Creates a new BackgroundCacheDownloader object.
     *
     * @param waiter The waiter that is used to wait on background tasks.
     * @param newsgroup The Newsgroup the cache is for.
     * @param client The NNTP client.
     * @param dialog The Wait Dialog.
     */
    public BackgroundCacheDownloader(View waiter, Newsgroup newsgroup,
                                     NNTPClient client, WaitDialog dialog) {
        this.waiter = waiter;
        this.newsgroup = newsgroup;
        this.client = client;
        this.dialog = dialog;
    }

    /**
     * Downloads and caches this group.
     */
    public void run() {
        try {
            MessageTree cache = newsgroup.getCache();
            MessageTree tree = new MessageTree();

            Enumeration enumeration = cache.preorderTraversal();
            int count = 0;

            while (enumeration.hasMoreElements()) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumeration.nextElement();
                Object object = node.getUserObject();

                MessageHeader header = null;
                Message message = null;

                if (object != null) {
                    String text = "Downloading message " + (count++);
                    dialog.setText(text);

                    if (object instanceof MessageHeader) {
                        header = (MessageHeader)object;

                        String messageId = header.getValue("message-id");
                        message = this.client.getMessage(messageId, header);
                    } else if (object instanceof Message) {
                        message = (Message)object;
                        header = message.getHeader();
                    } else {
                        EventLogger.getInstance().logError("MessageTree node is neither Message nor " +
                                                           "MessageHeader in " +
                                                           "BackgroundCacheDownloader.run()...");
                    }

                    if (header.containsHeader("references")) {
                        String references = header.getValue("references");
                        StringTokenizer tokens = new StringTokenizer(references);

                        tree.addThreadChild(message, tokens);
                    } else {
                        tree.addChild(message);
                    }
                }
            }

            cache.transpose(tree);

            BackgroundCacheWriter writer = new BackgroundCacheWriter(newsgroup,
                                                                     cache);
            writer.start();

            waiter.addBackgroundThread(writer);
        } catch (Exception e) {
            EventLogger.getInstance().logError(e.toString() +
                                               " in BackgroundCacheDownloader.run()...");
        }

        finished = true;

        while (!dialog.isVisible()) {
            try {
                Thread.sleep(50);
            } catch (InterruptedException e) {
                // this is not a problem...
            }
        }

        dialog.setVisible(false);
    }

    /**
     * Returns a String representation of this BackgroundCacheDownloader.
     *
     * @return A String representation.
     */
    public String toString() {
        return "BackgroundCacheDownloader[waiter=" + waiter +
               ", newsgroup=" + newsgroup + ", client=" + client +
               ", wait dialog=" + dialog + "]";
    }
}
