/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.tools;

import java.io.File;

import java.util.Arrays;
import java.util.StringTokenizer;

import nan.model.Message;
import nan.model.MessageHeader;
import nan.model.MessageTree;
import nan.model.Newsgroup;
import nan.model.XMLFactory;

import nan.view.WaitDialog;

import org.dom4j.Document;

/**
 * Reads a message cache in the background.
 *
 * @author John David Ratliff
 * @version 1.8, 1/6/04
 */
public class BackgroundCacheReader extends AbstractBackgroundTask {
    private File directory;
    private Newsgroup newsgroup;
    private MessageTree tree;
    private WaitDialog dialog;

    /**
     * Creates a new BackgroundCacheReader object.
     *
     * @param directory The directory of the cache.
     * @param newsgroup The Newsgroup we are reading.
     * @param tree The tree to add the cache to.
     * @param dialog The dialog to display progress on.
     */
    public BackgroundCacheReader(File directory, Newsgroup newsgroup,
                                 MessageTree tree, WaitDialog dialog) {
        this.directory = directory;
        this.newsgroup = newsgroup;
        this.tree = tree;
        this.dialog = dialog;
    }

    /**
     * Runs this background task.
     */
    public void run() {
        try {
            File[] files = directory.listFiles();

            if (files != null) {
                Arrays.sort(files);

                for (int i = 0; i < files.length; i++) {
                    String text = "Reading message " + (i + 1) + " of " +
                                  files.length + ".";
                    dialog.setText(text);

                    Document document = XMLSerializer.readDocument(files[i]);
                    MessageHeader header = null;
                    Message message = null;

                    String rootName = document.getRootElement().getName();

                    if (rootName.equals(Newsgroup.XML_TAG)) {
                        Newsgroup newsgroup = XMLFactory.getNewsgroup(document);
                        this.newsgroup.transpose(newsgroup);
                    } else {
                        if (rootName.equals(Message.XML_TAG)) {
                            message = XMLFactory.getMessage(document);
                            header = message.getHeader();
                        } else if (rootName.equals(MessageHeader.XML_TAG)) {
                            header = XMLFactory.getMessageHeader(document);
                        } else {
                            EventLogger.getInstance().logError("Cache file is not a Newsgroup, " +
                                                               "Message, or MessageHeader!");
                        }

                        if (header.containsHeader("references")) {
                            String references = header.getValue("references");
                            StringTokenizer tokens = new StringTokenizer(references);

                            if (message != null) {
                                tree.addThreadChild(message, tokens);
                            } else {
                                tree.addThreadChild(header, tokens);
                            }
                        } else {
                            if (message != null) {
                                tree.addChild(message);
                            } else {
                                tree.addChild(header);
                            }
                        }
                    }
                }
            }
        } catch (Exception e) {
            // cannot let an exception crash the program
            EventLogger.getInstance().logError(e.toString() +
                                               " during BackgroundCacheReader.run()...");
        }

        // make sure we don't leave before the dialog is displayed
        // that could halt the program...
        while (!dialog.isVisible()) {
            try {
                Thread.sleep(50);
            } catch (InterruptedException e) {
                // not a problem...
            }
        }

        finished = true;
        dialog.setVisible(false);
    }

    /**
     * Returns a String representation of this object.
     *
     * @return A String representation.
     */
    public String toString() {
        return "BackgroundCacheReader[directory=" + directory + ", tree=" +
               tree + ", dialog=" + this.dialog + "]";
    }
}
