/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.tools;

import java.io.File;

import java.util.Enumeration;

import javax.swing.tree.DefaultMutableTreeNode;

import nan.NaN;

import nan.model.Message;
import nan.model.MessageHeader;
import nan.model.MessageTree;
import nan.model.Newsgroup;
import nan.model.XMLSerializable;

/**
 * Writes a message cache to disk in the background.
 *
 * @author John David Ratliff
 * @version 1.22, 1/6/04
 */
public class BackgroundCacheWriter extends AbstractBackgroundTask {
    private Newsgroup newsgroup;
    private MessageTree tree;

    /**
     * Creates a new BackgroundCacheWriter object.
     *
     * @param newsgroup The Newsgroup to cache.
     * @param tree The MessageTree with the messages.
     */
    public BackgroundCacheWriter(Newsgroup newsgroup, MessageTree tree) {
        this.newsgroup = newsgroup;
        this.tree = tree;
    }

    /**
     * Gets the cache directory for a Newsgroup.
     *
     * @param newsgroup The Newsgroup.
     *
     * @return The cache directory File.
     */
    public static File getCacheDirectory(Newsgroup newsgroup) {
        return XMLSerializer.getFilePath(NaN.CACHE_DIRECTORY,
                                         newsgroup.getName());
    }

    /**
     * Gets the cache file for a Newsgroup Message.
     *
     * @param newsgroup The Newsgroup.
     * @param header The MessageHeader of the Message.
     *
     * @return The cache File.
     */
    public static File getCacheFile(Newsgroup newsgroup, MessageHeader header) {
        File directory = getCacheDirectory(newsgroup);
        String article = header.getValue("X-Article-Number");

        // pad to 6 digits for proper lexiographic comparison
        // 2 must not be > than 10... 02 < 10!
        while (article.length() < 6) {
            article = "0" + article;
        }

        return new File(directory.getAbsolutePath() + File.separator +
                        article + ".xml");
    }

    /**
     * Gets the group cache file.
     *
     * @param newsgroup The newsgroup.
     *
     * @return The group cache file.
     */
    public static File getGroupFile(Newsgroup newsgroup) {
        File directory = getCacheDirectory(newsgroup);
        return new File(directory.getAbsolutePath() + File.separator +
                        "group.xml");
    }

    /**
     * Writes a message to the cache.
     *
     * @param newsgroup The Newsgroup.
     * @param xml The XMLSerializable object.
     * @param header The MessageHeader of this cache file.
     */
    public static void writeCacheFile(Newsgroup newsgroup,
                                      XMLSerializable xml,
                                      MessageHeader header) {
        File file = getCacheFile(newsgroup, header);
        XMLSerializer.write(xml.toXML(), file);
    }

    /**
     * Runs the background task.
     */
    public void run() {
        try {
            File directory = getCacheDirectory(newsgroup);

            if (!directory.exists()) {
                directory.mkdir();
            }

            // write the group file
            File file = getGroupFile(newsgroup);
            XMLSerializer.write(newsgroup.toXML(), file);

            Enumeration enumeration = this.tree.preorderTraversal();

            while (enumeration.hasMoreElements()) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumeration.nextElement();
                Object object = node.getUserObject();

                if (object != null) {
                    MessageHeader header = null;

                    if (object instanceof MessageHeader) {
                        header = (MessageHeader)object;
                    } else if (object instanceof Message) {
                        header = ((Message)object).getHeader();
                    } else {
                        EventLogger.getInstance().logError("User object is not of type Message or " +
                                                           "MessageHeader in " +
                                                           "BackgroundCacheWriter.run()...");
                    }

                    writeCacheFile(newsgroup, (XMLSerializable)object, header);
                }
            }
        } catch (Exception e) {
            // cannot let an Exception crash the program...
            EventLogger.getInstance().logError(e.toString() +
                                               "during BackgroundCacheWriter.run()...");
        }

        // mark the task finished
        finished = true;
    }

    /**
     * Returns a String representation of this object.
     *
     * @return A String representation.
     */
    public String toString() {
        return "BackgroundCacheWriter[newsgroup=" + this.newsgroup +
               ", tree=" + this.tree + "]";
    }
}
