/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.tools;

import java.util.StringTokenizer;

import nan.exception.nntp.InvalidArticleException;

import nan.model.MessageHeader;
import nan.model.MessageTree;
import nan.model.Newsgroup;

import nan.nntp.NNTPClient;

import nan.view.WaitDialog;

/**
 * Downloads Newsgroup cache into MessageTree in a background thread.
 *
 * @author John David Ratliff
 * @version 1.9, 1/6/04
 */
public class BackgroundNewsgroupDownloader extends AbstractBackgroundTask {
    private NNTPClient client;
    private MessageTree tree;
    private Newsgroup newsgroup;
    private WaitDialog dialog;

    /**
     * Creates a new BackgroundNewsgroupDownloader object.
     *
     * @param client The NNTPClient to retreive messages from.
     * @param dialog The dialog to display progress on.
     * @param newsgroup The newsgroup to download messages from.
     * @param tree The tree to add the messages to.
     */
    public BackgroundNewsgroupDownloader(NNTPClient client,
                                         WaitDialog dialog,
                                         Newsgroup newsgroup,
                                         MessageTree tree) {
        this.client = client;
        this.dialog = dialog;
        this.newsgroup = newsgroup;
        this.tree = tree;
    }

    /**
     * Runs the task in the background.
     */
    public void run() {
        newsgroup.setUpdated();

        int first = newsgroup.getFirstArticle();
        int last = newsgroup.getLastArticle();
        int total = last - first + 1;
        int messages = newsgroup.getMessages();

        if (messages != Newsgroup.ALL_MESSAGES) {
            first = Math.max(newsgroup.getFirstArticle(),
                             (last - messages + 1));
            total = last - first + 1;
        }

        try {
            for (int i = first, j = 1; i <= last; i++, j++) {
                String text = "Downloading header for message " + j +
                              " of " + total + ".";
                dialog.setText(text);

                try {
                    MessageHeader header = client.getHeader(i);

                    if (header.containsHeader("references")) {
                        String references = header.getValue("references");
                        StringTokenizer tokens = new StringTokenizer(references);

                        tree.addThreadChild(header, tokens);
                    } else {
                        tree.addChild(header);
                    }
                } catch (InvalidArticleException e) {
                    // this could conceivably happen over a deleted message
                    EventLogger.getInstance().logMessage("InvalidArticleException caught in " +
                                                         "BackgroundNewsgroupDownloader.run()...");
                }
            }

            newsgroup.setNextArticle(last + 1);
        } catch (Exception e) {
            // unexpected errors cannot be allowed to crash the program...
            EventLogger.getInstance().logError(e.toString() +
                                               " during BackgroundNewsgroupDownloader.run()...");
        }

        // make sure we don't leave before the dialog is displayed
        // that could halt the program...
        while (!dialog.isVisible()) {
            try {
                Thread.sleep(50);
            } catch (InterruptedException e) {
                // not a problem...
            }
        }

        // we are finished
        finished = true;
        dialog.setVisible(false);
    }

    /**
     * Returns a String representation of this object.
     *
     * @return A String representation.
     */
    public String toString() {
        return "BackgroundNewsgroupDownloader[client=" + this.client +
               ", dialog=" + this.dialog + ", newsgroup=" + this.newsgroup +
               ", tree=" + this.tree + "]";
    }
}
