/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.tools;

import nan.exception.UnfinishedException;

import nan.model.NewsgroupList;

import nan.nntp.NNTPClient;

import nan.tools.EventLogger;

import nan.view.WaitDialog;

/**
 * Reads the available NewsgroupList from an NNTP Server in a background
 * thread.
 *
 * @author John David Ratliff
 * @version 1.11, 1/6/03
 */
public class BackgroundNewsgroupListDownloader extends AbstractBackgroundTask {
    private NNTPClient client;
    private WaitDialog dialog;
    private NewsgroupList list;

    /**
     * Creates a new ReadNewsgroupsThread object.
     *
     * @param dialog The WaitDialog being displayed while we wait.
     * @param client The NNTP client.
     */
    public BackgroundNewsgroupListDownloader(WaitDialog dialog,
                                             NNTPClient client) {
        super();

        this.dialog = dialog;
        this.client = client;
    }

    /**
     * Gets the list of Newsgroups read from the server.
     *
     * @return The NewsgroupList.
     *
     * @throws UnfinishedException if the task hasn't finished.
     */
    public NewsgroupList getNewsgroupList() throws UnfinishedException {
        if (!isFinished()) {
            throw new UnfinishedException();
        }

        return this.list;
    }

    /**
     * Runs the thread task.
     */
    public void run() {
        try {
            if (this.client.isConnected()) {
                this.list = this.client.getNewsgroupList();
            }
        } catch (Exception e) {
            // don't let an exception crash the program
            EventLogger.getInstance().logError(e.toString() +
                                               " during ReadNewsgroupsThread.run()...");
        }

        // make sure we don't leave before the dialog is displayed
        // that could halt the program...
        while (!dialog.isVisible()) {
            try {
                Thread.sleep(50);
            } catch (InterruptedException e) {
                // not a problem...
            }
        }

        this.finished = true;
        this.dialog.setVisible(false);
    }

    /**
     * Returns a String representation of this object.
     *
     * @return A String representation.
     */
    public String toString() {
        return "BackgroundNewsgroupListDownloader[NNTP client=" +
               this.client + ", Newsgroup list=" + this.list +
               ", wait dialog=" + this.dialog + "]";
    }
}
