/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.tools;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.StringReader;

import nan.tools.EventLogger;

/**
 * A class to handle encoding schemes used with NNTP messages. The uuencode
 * and uudecode functions are based on the GNU sharutils uudecode.c and
 * uuencode.c sources.
 *
 * @author John David Ratliff
 * @version 1.2, 1/6/04
 */
public class Encoder {
    private static final char[] UU_ALPHABET = {
                                                  '`', '!', '"', '#', '$',
                                                  '%', '&', '\'', '(', ')',
                                                  '*', '+', ',', '-', '.',
                                                  '/', '0', '1', '2', '3',
                                                  '4', '5', '6', '7', '8',
                                                  '9', ':', ';', '<', '=',
                                                  '>', '?', '@', 'A', 'B',
                                                  'C', 'D', 'E', 'F', 'G',
                                                  'H', 'I', 'J', 'K', 'L',
                                                  'M', 'N', 'O', 'P', 'Q',
                                                  'R', 'S', 'T', 'U', 'V',
                                                  'W', 'X', 'Y', 'Z', '[',
                                                  '\\', ']', '^', '_'
                                              };

    /**
     * Creates a new Encoder object.
     */
    private Encoder() {
        // static method class
        // no instantation please...
    }

    /**
     * Encodes a character in the UU encoding scheme.
     *
     * @param b The byte to encode.
     *
     * @return The UU encoded character.
     */
    private static char uuencode(byte b) {
        b = (byte)(b & 077);

        return UU_ALPHABET[b];
    }

    /**
     * UUencodes a block of data.
     *
     * @param data The data.
     *
     * @return The UU encoded String.
     */
    public static String uuencode(byte[] data) {
        StringBuffer string = new StringBuffer();
        ByteArrayInputStream input = new ByteArrayInputStream(data);

        // this source is a bit complicated
        // I don't even fulle understand it
        // it's a C to Java translation of uuencode.c
        // from the GNU sharutils 4.2.1
        int n;
        int p = 0;
        char ch;
        byte[] buffer;

        for (;;) {
            n = 0;
            buffer = new byte[80];

            do {
                int length = input.read(buffer, 0, 45);

                if (length <= 0) {
                    break;
                }

                n += length;
            } while (n < 45);

            if (n == 0) {
                break;
            }

            ch = uuencode((byte)n);
            string.append(ch);

            p = 0;

            for (int temp; n > 2; n -= 3, p += 3) {
                temp = buffer[p] >> 2;
                ch = uuencode((byte)temp);
                string.append(ch);

                temp = ((buffer[p] << 4) & 060) |
                       ((buffer[p + 1] >> 4) & 017);
                ch = uuencode((byte)temp);
                string.append(ch);

                temp = ((buffer[p + 1] << 2) & 074) |
                       ((buffer[p + 2] >> 6) & 03);
                ch = uuencode((byte)temp);
                string.append(ch);

                temp = buffer[p + 2] & 077;
                ch = uuencode((byte)temp);
                string.append(ch);
            }

            if (n != 0) {
                break;
            }

            string.append('\n');
        }

        while (n != 0) {
            int c1 = (int)buffer[p];
            int c2 = (n == 1) ? 0 : buffer[p + 1];

            int temp;

            temp = c1 >> 2;
            ch = uuencode((byte)temp);
            string.append(ch);

            temp = ((c1 << 4) & 060) | ((c2 >> 4) & 017);
            ch = uuencode((byte)temp);
            string.append(ch);

            if (n == 1) {
                ch = uuencode((byte)0);
            } else {
                temp = (c2 << 2) & 074;
                ch = uuencode((byte)temp);
            }

            string.append(ch);

            ch = uuencode((byte)0);
            string.append(ch);

            string.append('\n');

            break;
        }

        ch = uuencode((byte)0);
        string.append(ch);
        string.append('\n');

        return string.toString();
    }

    /**
     * Decodes a character in UU encoding.
     *
     * @param ch The encoded character.
     *
     * @return The decoded byte.
     */
    private static byte uudecode(char ch) {
        return (byte)((ch - ' ') & 077);
    }

    /**
     * UUdecodes a UUencoded String.
     *
     * @param data The UUencoded String.
     *
     * @return The decoded data.
     */
    public static byte[] uudecode(String data) {
        ByteArrayOutputStream output = new ByteArrayOutputStream();

        // this source is a bit complicated
        // I don't even fulle understand it
        // it's a C to Java translation of uudecode.c
        // from the GNU sharutils 4.2.1
        try {
            BufferedReader reader = new BufferedReader(new StringReader(data));
            String line;

            int n;
            int p;

            while ((line = reader.readLine()) != null) {
                char[] buffer = line.toCharArray();
                char ch;

                p = 0;
                n = uudecode(buffer[p++]);

                if (n > 0) {
                    for (int temp; n > 0; p += 4, n -= 3) {
                        if (n >= 3) {
                            temp = (uudecode(buffer[p]) << 2) |
                                   (uudecode(buffer[p + 1]) >> 4);
                            output.write(temp);

                            temp = (uudecode(buffer[p + 1]) << 4) |
                                   (uudecode(buffer[p + 2]) >> 2);
                            output.write(temp);

                            temp = (uudecode(buffer[p + 2]) << 6) |
                                   (uudecode(buffer[p + 3]));
                            output.write(temp);
                        } else {
                            if (n >= 1) {
                                temp = (uudecode(buffer[p]) << 2) |
                                       (uudecode(buffer[p + 1]) >> 4);
                                output.write(temp);
                            }

                            if (n >= 2) {
                                temp = (uudecode(buffer[p + 1]) << 4) |
                                       (uudecode(buffer[p + 2]) >> 2);
                                output.write(temp);
                            }
                        }
                    }
                }
            }

            reader.close();
            output.close();
        } catch (IOException e) {
            EventLogger.getInstance().logError("IOException during Encoder.uudecode()...");
        }

        return output.toByteArray();
    }
}
