/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.tools;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

/**
 * A program to count the number of files and lines per file of a directory
 * recursively.
 *
 * @author John David Ratliff
 * @version 1.4, 1/2/04
 */
public class LineCounter {
    /**
     * Gets the count of files and lines in a directory.
     *
     * @param directory The directory to inventory.
     *
     * @return The DirectoryResult.
     */
    private static DirectoryResult getCount(File directory) {
        DirectoryResult result = new DirectoryResult();

        File[] files = directory.listFiles();

        if (files != null) {
            for (int i = 0; i < files.length; i++) {
                if (files[i].isDirectory()) {
                    DirectoryResult temp = getCount(files[i]);

                    result.lines += temp.lines;
                    result.files += temp.files;
                } else {
                    result.files++;

                    try {
                        BufferedReader reader = new BufferedReader(new FileReader(files[i]));
                        String line;

                        while ((line = reader.readLine()) != null) {
                            result.lines++;
                        }

                        reader.close();
                    } catch (IOException e) {
                        // this shouldn't happen at all...
                        e.printStackTrace();
                    }
                }
            }
        }

        return result;
    }

    /**
     * Called to start the program.
     *
     * @param args The first argument is the directory.
     */
    public static void main(String[] args) {
        if (args.length == 0) {
            System.out.println("usage: java LineCounter base_directory");
            System.exit(-1);
        }

        File directory = new File(args[0]);
        DirectoryResult result = getCount(directory);

        System.out.println("The directory has " + result.files +
                           " files containing " + result.lines +
                           " lines of code.");
    }

    /**
     * The result of a directory inventory.
     *
     * @author John David Ratliff
     * @version 1.2, 1/2/04
     */
    private static class DirectoryResult {
        /**
         * How many lines are in the files.
         */
        public long lines;

        /**
         * How many files are in this directory.
         */
        public long files;

        /**
         * Returns a String representation of this result.
         *
         * @return A String representation.
         */
        public String toString() {
            return "DirectoryResult[files=" + files + ", lines=" + lines +
                   "]";
        }
    }
}
