/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.tools;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import nan.NaN;

/**
 * Handles archival functions so the settings can be remote saved easily.
 *
 * @author John David Ratliff
 * @version 1.3, 1/3/04
 */
public class SettingsArchiver {
    /**
     * Creates a new SettingsArchiver object.
     */
    private SettingsArchiver() {
        // this is just a helper method class
        // no instantiation please...
    }

    /**
     * Writes a file to the Zip archive.
     *
     * @param output The ZipOutputStream.
     * @param file The File to write.
     *
     * @throws IOException if any problems with writing occur.
     */
    private static void writeFile(ZipOutputStream output, File file)
                           throws IOException {
        FileInputStream input = new FileInputStream(file);

        byte[] buffer = new byte[8192];
        int length;

        while ((length = input.read(buffer, 0, 8192)) > 0) {
            output.write(buffer, 0, length);
        }

        input.close();
    }

    /**
     * Adds a directory to the Zip archive.
     *
     * @param output The ZipOutputStream.
     * @param directory The directory to add.
     * @param prefix The path to the directory.
     * @param dirPrefix The child path of the prefix to the current
     *        directory.
     *
     * @throws IOException if any problems occur during writing.
     */
    private static void addDirectory(ZipOutputStream output, File directory,
                                     String prefix, String dirPrefix)
                              throws IOException {
        String[] filenames = directory.list();

        if (filenames != null) {
            for (int i = 0; i < filenames.length; i++) {
                String name = dirPrefix + filenames[i];
                File file = new File(prefix, name);

                if (file.isDirectory()) {
                    // add the directory entry
                    ZipEntry entry = new ZipEntry(name + '/');
                    output.putNextEntry(entry);

                    addDirectory(output, file, prefix,
                                 (dirPrefix + file.getName() +
                                 File.separatorChar));
                } else {
                    // add the file entry
                    ZipEntry entry = new ZipEntry(name);
                    output.putNextEntry(entry);

                    // add the file
                    writeFile(output, file);
                }
            }
        }
    }

    /**
     * Archives the settings to the Nan ARCHIVE_FILE.
     */
    public static void archive() {
        File file = NaN.ARCHIVE_FILE;

        try {
            ZipOutputStream output = new ZipOutputStream(new FileOutputStream(file));
            ZipEntry entry;

            // add the available group list
            File next = XMLSerializer.getFilePath(NaN.SETTINGS_DIRECTORY,
                                                  NaN.AVAILABLE_NEWSGROUPS_FILE);

            if (next.exists()) {
                entry = new ZipEntry(NaN.SETTINGS_DIRECTORY +
                                     File.separatorChar +
                                     NaN.AVAILABLE_NEWSGROUPS_FILE);
                output.putNextEntry(entry);

                writeFile(output, next);
            }

            // add the subscribed group list
            next = XMLSerializer.getFilePath(NaN.SETTINGS_DIRECTORY,
                                             NaN.SUBSCRIBED_NEWSGROUPS_FILE);

            if (next.exists()) {
                entry = new ZipEntry(NaN.SETTINGS_DIRECTORY +
                                     File.separatorChar +
                                     NaN.SUBSCRIBED_NEWSGROUPS_FILE);
                output.putNextEntry(entry);

                writeFile(output, next);
            }

            // add the user settings
            next = XMLSerializer.getFilePath(NaN.SETTINGS_DIRECTORY,
                                             NaN.USER_SETTINGS_FILE);

            if (next.exists()) {
                entry = new ZipEntry(NaN.SETTINGS_DIRECTORY +
                                     File.separatorChar +
                                     NaN.USER_SETTINGS_FILE);
                output.putNextEntry(entry);

                writeFile(output, next);
            }

            // add the cache directory
            File base = new File(NaN.BASE_DIRECTORY);
            File cache = new File(base, NaN.CACHE_DIRECTORY);
            String dirPrefix = NaN.CACHE_DIRECTORY + File.separatorChar;

            addDirectory(output, cache, base.getAbsolutePath(), dirPrefix);

            // close the zip file
            output.close();
        } catch (FileNotFoundException e) {
            // path problem. this shouldn't happen..
            EventLogger.getInstance().logError("FileNotFoundException during " +
                                               "SettingsArchiver.archive()...");
        } catch (IOException e) {
            // this shouldn't happen too often...
            EventLogger.getInstance().logError("IOException during " +
                                               "SettingsArchiver.archive()...");
        }
    }

    /**
     * Reads and extracts a zip archive.
     *
     * @param input The ZipInputStream.
     * @param directory The directory to extract the files to.
     *
     * @throws IOException if any problems occur while reading.
     */
    private static void readZip(ZipInputStream input, File directory)
                         throws IOException {
        ZipEntry entry;

        directory.mkdir();

        while ((entry = input.getNextEntry()) != null) {
            File file = new File(directory, entry.getName());

            if (entry.isDirectory()) {
                file.mkdir();
            } else {
                FileOutputStream output = new FileOutputStream(file);
                byte[] buffer = new byte[8192];
                int length;

                while ((length = input.read(buffer, 0, 8192)) > 0) {
                    output.write(buffer, 0, length);
                }

                output.close();
            }
        }
    }

    /**
     * Unarchives the NaN ARCHIVE_FILE.
     */
    public static void unarchive() {
        try {
            ZipInputStream input = new ZipInputStream(new FileInputStream(NaN.ARCHIVE_FILE));
            readZip(input, new File(NaN.BASE_DIRECTORY));
            input.close();
        } catch (FileNotFoundException e) {
            // this should hopefully never happen...
            EventLogger.getInstance().logError("FileNotFoundException during " +
                                               "SettingsArchiver.unarchive()...");
        } catch (IOException e) {
            EventLogger.getInstance().logError("IOException during " +
                                               "SettingsArchiver.unarchive()...");
        }
    }
}
