/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.tools;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import nan.NaN;

import nan.exception.XMLParseException;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;

import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;

/**
 * A helper class to handle file serialization of XML Documents.
 *
 * @author John David Ratliff
 * @version 1.7, 12/31/03
 */
public class XMLSerializer {
    private static final OutputFormat PRETTY = buildOutputFormat();

    /**
     * Creates a new XMLSerializer object.
     */
    private XMLSerializer() {
        // this is a helper method class. no instantiation please.
    }

    /**
     * Builds the output format used in the XML serializer.
     *
     * @return The OutputFormat.
     */
    private static OutputFormat buildOutputFormat() {
        OutputFormat format = OutputFormat.createPrettyPrint();
        format.setTrimText(false);

        return format;
    }

    /**
     * Gets a File from a directory and filename.
     *
     * @param directory The directory under the NaN directory.
     * @param filename The filename of the file.
     *
     * @return The File with full path.
     */
    public static File getFilePath(String directory, String filename) {
        String path = NaN.BASE_DIRECTORY + File.separator + directory +
                      File.separator + filename;

        return new File(path);
    }

    /**
     * Writes an XML document to a file.
     *
     * @param document The XML Document.
     * @param file The file.
     */
    public static void write(Document document, File file) {
        try {
            FileWriter writer = new FileWriter(file);
            XMLWriter xmlWriter = new XMLWriter(writer, PRETTY);
            xmlWriter.write(document);
            xmlWriter.close();
            writer.close();
        } catch (IOException e) {
            EventLogger.getInstance().logError("IOException during XMLSerializer.write() to " +
                                               file.getName() + "...");
        }
    }

    /**
     * Reads a text file from the disk.
     *
     * @param file The File to read.
     *
     * @return A String with all the files text.
     *
     * @throws IOException if an error occurs during reading.
     */
    public static String readFile(File file) throws IOException {
        StringBuffer buffer = new StringBuffer((int)file.length());

        try {
            FileReader reader = new FileReader(file);
            char[] temp = new char[8192];

            for (int i = 0; i < (int)file.length(); i += 8192) {
                int read = reader.read(temp, 0, 8192);
                buffer.append(temp, 0, read);
            }
        } catch (FileNotFoundException e) {
            throw e;
        } catch (IOException e) {
            EventLogger.getInstance().logError("IOException during XMLSerializer.read(" +
                                               file.getName() + ")...");

            throw e;
        }

        return buffer.toString();
    }

    /**
     * Reads an XML Document from a file.
     *
     * @param file The file.
     *
     * @return The XML Document.
     *
     * @throws XMLParseException if the file cannot be parsed.
     */
    public static Document readDocument(File file) throws XMLParseException {
        try {
            return DocumentHelper.parseText(readFile(file));
        } catch (IOException e) {
            throw new XMLParseException();
        } catch (DocumentException e) {
            EventLogger.getInstance().logError("DocumentException during XMLSerializer.read(" +
                                               file.getName() + ")...");

            throw new XMLParseException();
        }
    }
}
