/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.view;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.WindowConstants;

import nan.tools.EventLogger;

/**
 * A dialog for license acceptance.
 *
 * @author John David Ratliff
 * @version 1.5, 12/29/03
 */
public class LicenseDialog extends JDialog implements ActionListener {
    private boolean accepted;
    private JButton acceptButton;
    private JPanel buttonPanel;
    private JButton cancelButton;
    private JTextArea license;
    private JLabel licenseLabel;
    private JScrollPane licenseScroller;
    private static final String ACCEPT_ACTION = "_accept";
    private static final String CANCEL_ACTION = "_cancel";

    /**
     * Creates a new LicenseDialog object.
     *
     * @param parent The parent of this dialog.
     * @param accept If this should be an acceptance dialog.
     */
    public LicenseDialog(Frame parent, boolean accept) {
        super(parent, true);
        initComponents(accept);
    }

    /**
     * Initializes the dialog components.
     *
     * @param accept If this should be an acceptance dialog.
     */
    private void initComponents(boolean accept) {
        GridBagConstraints gridBagConstraints;

        licenseLabel = new JLabel();
        licenseScroller = new JScrollPane();
        license = new JTextArea();
        buttonPanel = new JPanel();
        acceptButton = new JButton();
        cancelButton = new JButton();

        getContentPane().setLayout(new GridBagLayout());

        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("License Agreement");
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent event) {
                    cancelAction();
                }
            });

        licenseLabel.setText("Do you accept the terms of this license agreement?");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new Insets(5, 0, 5, 0);
        getContentPane().add(licenseLabel, gridBagConstraints);

        license.setEditable(false);
        license.setFont(new Font("Monospaced", 0, 12));
        licenseScroller.setViewportView(license);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(5, 10, 5, 10);
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(licenseScroller, gridBagConstraints);

        if (accept) {
            acceptButton.setMnemonic('a');
            acceptButton.setText("Accept");
        } else {
            acceptButton.setMnemonic('o');
            acceptButton.setText("Okay");
        }

        acceptButton.setActionCommand(ACCEPT_ACTION);
        acceptButton.addActionListener(this);
        buttonPanel.add(acceptButton);

        if (accept) {
            cancelButton.setMnemonic('c');
            cancelButton.setText("Cancel");
        } else {
            cancelButton.setMnemonic('c');
            cancelButton.setText("Close");
        }

        cancelButton.setActionCommand(CANCEL_ACTION);
        cancelButton.addActionListener(this);
        buttonPanel.add(cancelButton);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new Insets(5, 0, 5, 0);
        getContentPane().add(buttonPanel, gridBagConstraints);

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width - 640) / 2,
                  (screenSize.height - 480) / 2, 640, 480);
    }

    /**
     * Called when the accept button is pressed.
     */
    private void acceptAction() {
        accepted = true;
        setVisible(false);
    }

    /**
     * Called when the cancel button is pressed.
     */
    private void cancelAction() {
        accepted = false;
        setVisible(false);
    }

    /**
     * Asks if the license terms were accepted.
     *
     * @return true if the license was accepted; false otherwise.
     */
    public boolean wasAccepted() {
        return this.accepted;
    }

    /**
     * Displays this dialog with a license.
     *
     * @param license The license.
     */
    public void display(String license) {
        accepted = false;
        this.license.setText(license);
        this.license.setCaretPosition(0);
        setVisible(true);
    }

    /**
     * The registered callback for ActionEvents on this dialog.
     *
     * @param event The ActionEvent that triggered this callback.
     */
    public void actionPerformed(ActionEvent event) {
        String action = event.getActionCommand();

        if (action.equals(ACCEPT_ACTION)) {
            acceptAction();
        } else if (action.equals(CANCEL_ACTION)) {
            cancelAction();
        } else {
            EventLogger.getInstance().logError("Unknown action command " +
                                               action +
                                               " seen in LicenseDialog.actionPerformed()...");
        }
    }

    /**
     * Returns a String representation of this dialog.
     *
     * @return A String representation.
     */
    public String toString() {
        return "LicenseDialog[license label=" + licenseLabel + ", license=" +
               license + ", accept button=" + acceptButton +
               ", cancel button=" + cancelButton + "]";
    }
}
