/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import java.io.File;

import java.util.Date;

import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;

import javax.swing.table.TableColumnModel;

import com.sun.treetable.JTreeTable;

import nan.controller.AttachmentPopupMenu;

import nan.model.Attachment;
import nan.model.Message;
import nan.model.MessageHeader;
import nan.model.MessageTree;
import nan.model.MessageTreeTableModel;

import nan.tools.BackgroundAttachmentSaver;
import nan.tools.EventLogger;
import nan.tools.NNTPDateFormat;

/**
 * The main display of the application.
 *
 * @author John David Ratliff
 * @version 1.31, 1/7/04
 */
public class MessagePanel extends JPanel implements ActionListener,
                                                    MouseListener {
    private JLabel date;
    private JLabel dateLabel;
    private JLabel from;
    private JLabel fromLabel;
    private JLabel messagesLabel;
    private JPanel messageInfoPanel;
    private JEditorPane messagePane;
    private JScrollPane messageScroller;
    private JTreeTable messageTreeTable;
    private JScrollPane messageTreeScroller;
    private JList newsgroupList;
    private JScrollPane newsgroupScroller;
    private JLabel subject;
    private JLabel subjectLabel;
    private JLabel subscribedLabel;
    private JButton attachmentsButton;
    private Message message;
    private AttachmentPopupMenu attachments;
    private View parent;
    private static final MessageTree EMPTY_TREE = new MessageTree();

    /**
     * Creates a new MessagePanel object.
     *
     * @param parent The parent of this panel.
     * @param model The ListModel for the newsgroup list.
     */
    public MessagePanel(View parent, ListModel model) {
        this.parent = parent;

        initComponents(model);
    }

    /**
     * Initalizes the panel components.
     *
     * @param model The ListModel for the newsgroup list.
     */
    private void initComponents(ListModel model) {
        GridBagConstraints gridBagConstraints;

        newsgroupScroller = new JScrollPane();
        newsgroupList = new JList();
        messageTreeScroller = new JScrollPane();
        messageTreeTable = new JTreeTable(new MessageTreeTableModel());
        messageInfoPanel = new JPanel();
        fromLabel = new JLabel();
        from = new JLabel();
        subjectLabel = new JLabel();
        subject = new JLabel();
        dateLabel = new JLabel();
        date = new JLabel();
        messageScroller = new JScrollPane();
        messagePane = new JEditorPane();
        subscribedLabel = new JLabel();
        messagesLabel = new JLabel();
        attachmentsButton = new JButton();

        setLayout(new GridBagLayout());

        newsgroupList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        newsgroupList.setModel(model);
        newsgroupList.addListSelectionListener(this.parent);
        newsgroupList.addMouseListener(this.parent);
        newsgroupScroller.setPreferredSize(new Dimension(20, 100));
        newsgroupScroller.setViewportView(newsgroupList);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridheight = 3;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(5, 5, 5, 5);
        gridBagConstraints.weightx = 0.1;
        gridBagConstraints.weighty = 1.0;
        add(newsgroupScroller, gridBagConstraints);

        TableColumnModel columns = messageTreeTable.getColumnModel();
        JTreeTable.TreeTableCellRenderer renderer = (JTreeTable.TreeTableCellRenderer)messageTreeTable.getTree();

        columns.getColumn(0).setCellRenderer(new MessageTableCellRenderer(renderer));
        columns.getColumn(1).setCellRenderer(new SubjectTableCellRenderer());
        columns.getColumn(2).setCellRenderer(new DateTableCellRenderer());

        messageTreeTable.getTree().setRootVisible(false);
        messageTreeTable.getTree().addTreeSelectionListener(this.parent);
        messageTreeTable.addMouseListener(this.parent);
        messageTreeScroller.setPreferredSize(new Dimension(80, 40));
        messageTreeScroller.setViewportView(messageTreeTable);
        messageTreeScroller.getViewport().setBackground(Color.WHITE);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(5, 5, 5, 5);
        gridBagConstraints.weightx = 0.9;
        gridBagConstraints.weighty = 0.4;
        add(messageTreeScroller, gridBagConstraints);

        messageInfoPanel.setLayout(new GridBagLayout());

        messageInfoPanel.setPreferredSize(new Dimension(80, 20));
        fromLabel.setText("From: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new Insets(0, 5, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.05;
        messageInfoPanel.add(fromLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 0, 0, 5);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.95;
        messageInfoPanel.add(from, gridBagConstraints);

        attachmentsButton.setText("Attachments");
        attachmentsButton.setEnabled(false);
        attachmentsButton.addMouseListener(this);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridheight = 3;
        gridBagConstraints.insets = new Insets(0, 0, 0, 5);
        gridBagConstraints.weightx = 0.05;
        gridBagConstraints.weighty = 1.0;
        messageInfoPanel.add(attachmentsButton, gridBagConstraints);

        subjectLabel.setText("Subject: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.insets = new Insets(0, 5, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.05;
        messageInfoPanel.add(subjectLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 0, 0, 5);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.95;
        messageInfoPanel.add(subject, gridBagConstraints);

        dateLabel.setText("Date: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new Insets(0, 5, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 0.05;
        gridBagConstraints.weighty = 1.0;
        messageInfoPanel.add(dateLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 0.90;
        gridBagConstraints.weighty = 1.0;
        messageInfoPanel.add(date, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(0, 5, 0, 5);
        gridBagConstraints.weightx = 0.9;
        gridBagConstraints.weighty = 0.1;
        add(messageInfoPanel, gridBagConstraints);

        messageScroller.setPreferredSize(new Dimension(80, 40));
        messagePane.setEditable(false);
        messagePane.setFont(new Font("Monospaced", Font.PLAIN, 12));
        messageScroller.setViewportView(messagePane);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(0, 5, 5, 5);
        gridBagConstraints.weightx = 0.9;
        gridBagConstraints.weighty = 0.5;
        add(messageScroller, gridBagConstraints);

        subscribedLabel.setText("Subscribed Newsgroups");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new Insets(5, 0, 0, 0);
        add(subscribedLabel, gridBagConstraints);

        messagesLabel.setText("Newsgroup Messages");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new Insets(5, 0, 0, 0);
        add(messagesLabel, gridBagConstraints);
    }

    /**
     * Sets the MessageTree.
     *
     * @param tree The new MessageTree.
     */
    public void setMessageTree(MessageTree tree) {
        MessageTreeTableModel model = (MessageTreeTableModel)messageTreeTable.getTree()
                                                                             .getModel();
        model.setRoot(tree.getRoot());
        messageTreeTable.getTree().scrollRowToVisible(0);
    }

    /**
     * Displays a Message in the preview pane.
     *
     * @param message The Message.
     */
    public void display(Message message) {
        this.message = message;

        MessageHeader header = message.getHeader();

        from.setText(header.getValue("from"));
        subject.setText(header.getValue("subject"));
        messagePane.setText(message.getBody());
        messagePane.setCaretPosition(0);

        Date date = NNTPDateFormat.parseDate(header.getValue("date"));
        this.date.setText(NNTPDateFormat.toDisplayString(date));

        // if need be, build the popup menu
        if (message.getAttachmentCount() > 0) {
            this.attachments = new AttachmentPopupMenu(this);

            for (int i = 0; i < message.getAttachmentCount(); i++) {
                Attachment attachment = message.getAttachment(i);
                attachments.addAttachment(attachment.getName());
            }

            attachmentsButton.setEnabled(true);
        } else {
            attachmentsButton.setEnabled(false);
            attachments = null;
        }
    }

    /**
     * Clears the panel.
     */
    public void clear() {
        setMessageTree(EMPTY_TREE);
        from.setText("");
        subject.setText("");
        date.setText("");
        messagePane.setText("");
    }

    /**
     * Called when an action occurs on this component.
     *
     * @param event The event which triggered this callback.
     */
    public void actionPerformed(ActionEvent event) {
        String action = event.getActionCommand();

        if ((message != null) && (message.getAttachmentCount() > 0)) {
            boolean found = false;

            for (int i = 0; i < message.getAttachmentCount(); i++) {
                Attachment attachment = message.getAttachment(i);

                if (action.equals(attachment.getName())) {
                    i = message.getAttachmentCount();
                    found = true;

                    String title = "Save Attachment As...";
                    FileDialog dialog = new FileDialog(parent, title,
                                                       FileDialog.SAVE);
                    dialog.setFile(attachment.getName());
                    dialog.setVisible(true);

                    if (dialog.getFile() != null) {
                        File file = new File(dialog.getDirectory(),
                                             dialog.getFile());
                        BackgroundAttachmentSaver thread = new BackgroundAttachmentSaver(attachment,
                                                                                         file);
                        thread.start();

                        parent.addBackgroundThread(thread);
                    }
                }
            }

            if (!found) {
                EventLogger.getInstance().logError("Unknown action " +
                                                   action +
                                                   " in MessagePanel.actionPerformed()...");
            }
        } else {
            EventLogger.getInstance().logError("Unknown action " + action +
                                               " in MessagePanel.actionPerformed()...");
        }
    }

    /**
     * Responder to mouse click events.
     *
     * @param event The event which triggered this callback.
     */
    public void mouseClicked(MouseEvent event) {
        if ((event.getSource() == attachmentsButton) &&
            (attachmentsButton.isEnabled())) {
            attachments.show(attachmentsButton, event.getX(), event.getY());
        }
    }

    /**
     * Called when the mouse is released.
     *
     * @param event The event which triggered this callback.
     */
    public void mouseReleased(MouseEvent event) {
    }

    /**
     * Called when the mouse is pressed.
     *
     * @param event The event which triggered this callback.
     */
    public void mousePressed(MouseEvent event) {
    }

    /**
     * Called when the mouse enters an area.
     *
     * @param event The event which triggered this callback.
     */
    public void mouseEntered(MouseEvent event) {
    }

    /**
     * Called when the mouse exits an area.
     *
     * @param event The event which triggered this callback.
     */
    public void mouseExited(MouseEvent event) {
    }

    /**
     * Returns a String representation of this MessagePanel.
     *
     * @return A String representation.
     */
    public String toString() {
        return "MessagePanel[from=" + from + ", subject=" + subject +
               ", date=" + date + ",newsgroups list=" + newsgroupList +
               ", message tree table=" + messageTreeTable +
               ", message pane=" + messagePane + "]";
    }
}
