/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.view;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import java.io.File;
import java.io.IOException;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.StringTokenizer;

import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.WindowConstants;

import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;

import javax.swing.tree.DefaultMutableTreeNode;

import com.sun.treetable.JTreeTable;

import nan.NaN;

import nan.controller.Menu;
import nan.controller.MessagePopupMenu;
import nan.controller.NewsgroupPopupMenu;

import nan.exception.XMLParseException;

import nan.exception.nntp.InvalidArticleException;

import nan.model.License;
import nan.model.Message;
import nan.model.MessageHeader;
import nan.model.MessageTree;
import nan.model.Newsgroup;
import nan.model.NewsgroupList;
import nan.model.Settings;
import nan.model.XMLFactory;

import nan.nntp.NNTPClient;
import nan.nntp.NNTPClientImpl;

import nan.ssh.SSHConnection;

import nan.tools.BackgroundCacheDownloader;
import nan.tools.BackgroundCacheReader;
import nan.tools.BackgroundCacheWriter;
import nan.tools.BackgroundConnectThread;
import nan.tools.BackgroundDirectoryDeleter;
import nan.tools.BackgroundNewsgroupDownloader;
import nan.tools.BackgroundTask;
import nan.tools.BackgroundWriter;
import nan.tools.EventLogger;
import nan.tools.SettingsArchiver;
import nan.tools.XMLSerializer;

import org.dom4j.Document;

/**
 * The default View and main window of the NaN application.
 *
 * @author John David Ratliff
 * @version 1.96, 1/7/04
 */
public class View extends JFrame implements WindowListener,
                                            ListSelectionListener,
                                            TreeSelectionListener,
                                            MouseListener {
    private Menu menu = new Menu(this);
    private HelpFrame help;
    private Settings settings;
    private UserSettingsDialog userSettingsDialog;
    private ServerSettingsDialog serverSettingsDialog;
    private NewsgroupDialog newsgroupDialog;
    private MessagePanel panel;
    private NNTPClient client;
    private Newsgroup currentNewsgroup;
    private Message currentMessage;
    private NewsgroupPopupMenu newsgroupPopup;
    private MessagePopupMenu messagePopup;
    private List threads;

    /**
     * Creates a new View object.
     *
     * @param width The width of the window.
     * @param height The height of the window.
     */
    public View(int width, int height) {
        super(NaN.APP_NAME + ' ' + NaN.APP_VERSION);

        // setup frame properties
        setSize(width, height);
        setBounds(0, 0, width, height);
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        addWindowListener(this);
        setJMenuBar(menu);

        // open the event log
        EventLogger logger = EventLogger.getInstance();
        File file = XMLSerializer.getFilePath(NaN.LOG_DIRECTORY,
                                              EventLogger.getLogFileName());
        logger.open(file);

        // create the Help Frame
        this.help = new HelpFrame();

        // create the thread list
        this.threads = new ArrayList();

        // check and accept the license agreements
        this.checkLicenseAgreements();

        // create the NNTP client
        this.client = new NNTPClientImpl();

        // create the Settings
        this.settings = new Settings();

        // create the setting dialogs
        this.userSettingsDialog = new UserSettingsDialog(this,
                                                         settings.getUserSettings());
        this.serverSettingsDialog = new ServerSettingsDialog(this,
                                                             settings.getServerSettings());
        this.newsgroupDialog = new NewsgroupDialog(this);

        // create the popup menus
        this.newsgroupPopup = new NewsgroupPopupMenu(this);
        this.messagePopup = new MessagePopupMenu(this);

        // add the main panel
        this.panel = new MessagePanel(this,
                                      settings.getSubscribedNewsgroups()
                                              .getListModel());
        this.panel.setMessageTree(new MessageTree());

        getContentPane().add(this.panel);
    }

    /**
     * Called to exit the application.
     *
     * @param confirm If the exit should be confirmed or not.
     */
    public void exit(boolean confirm) {
        boolean exit = false;

        if (confirm) {
            String title = NaN.APP_NAME + ' ' + NaN.APP_VERSION;
            String message = "Really leaving?";

            int choice = JOptionPane.showConfirmDialog(this, message, title,
                                                       JOptionPane.YES_NO_OPTION,
                                                       JOptionPane.QUESTION_MESSAGE);

            if (choice == JOptionPane.YES_OPTION) {
                exit = true;
            }
        } else {
            exit = true;
        }

        if (exit) {
            try {
                // close the frame
                setVisible(false);

                SSHConnection ssh = SSHConnection.getInstance();

                if (settings.getServerSettings().useRemoteSave()) {
                    if (ssh.isConnected()) {
                        // remove save the settings
                        SettingsArchiver.archive();
                        ssh.upload(settings.getServerSettings().getDirectory(),
                                   NaN.ARCHIVE_FILE.getName());
                        NaN.ARCHIVE_FILE.delete();
                    }
                }

                // close the SSH connection
                ssh.close();

                // close the NNTP client
                this.client.disconnect();

                // wait on the other threads to finish
                for (int i = 0; i < this.threads.size(); i++) {
                    BackgroundTask task = (BackgroundTask)this.threads.get(i);

                    while (!task.isFinished()) {
                        try {
                            Thread.sleep(50);
                        } catch (InterruptedException e) {
                            // this is not a problem...
                        }
                    }
                }
            } catch (Exception e) {
                // no go crashie...
                EventLogger.getInstance().logError(e.toString() +
                                                   " in View.exit()...");
            }

            // close the event log
            EventLogger logger = EventLogger.getInstance();
            logger.close();

            System.exit(0);
        }
    }

    /**
     * Adds a background thread to the waiter. The program will wait for
     * each of these tasks for finish before calling exit.
     *
     * @param thread The BackgroundTask thread.
     */
    public void addBackgroundThread(BackgroundTask thread) {
        this.threads.add(thread);
    }

    /**
     * Gets the NNTPClient.
     *
     * @return The NNTPClient.
     */
    public NNTPClient getNNTPClient() {
        return this.client;
    }

    /**
     * Displays a license file.
     *
     * @param name The name of the license to display.
     */
    public void displayLicense(String name) {
        try {
            LicenseDialog dialog = new LicenseDialog(this, false);
            File file = new File(NaN.LICENSE_DIRECTORY + File.separator +
                                 name);
            String text = XMLSerializer.readFile(file);
            dialog.display(text);
            dialog.dispose();
        } catch (IOException e) {
            EventLogger.getInstance().logError("IOException caught during " +
                                               "View.displayLicense()...");
        }
    }

    /**
     * Displays a License agreement.
     *
     * @param dialog The LicenseDialog.
     * @param name The name of the license.
     * @param filename The filename of the license agreement.
     *
     * @return true if the license is accepted; false otherwise.
     */
    private boolean displayLicense(LicenseDialog dialog, String name,
                                   String filename) {
        File file = new File(NaN.LICENSE_DIRECTORY + File.separator + name);

        try {
            String text = XMLSerializer.readFile(file);
            dialog.display(text);

            if (dialog.wasAccepted()) {
                License license = new License(name, true, new Date());

                file = XMLSerializer.getFilePath(NaN.LICENSE_DIRECTORY,
                                                 filename);

                BackgroundWriter writer = new BackgroundWriter(license, file);
                writer.start();
                addBackgroundThread(writer);
            } else {
                return false;
            }
        } catch (IOException e) {
            EventLogger.getInstance().logError("IOException during " +
                                               "View.displayLicense()...");

            return false;
        }

        return true;
    }

    /**
     * Checks and displays the license agreements.
     */
    private void checkLicenseAgreements() {
        boolean needGPL = true;
        boolean needSource = true;
        boolean needEndUser = true;

        try {
            File file;
            Document document;
            License license;

            file = XMLSerializer.getFilePath(NaN.LICENSE_DIRECTORY,
                                             NaN.FREE_FILE);
            document = XMLSerializer.readDocument(file);
            license = XMLFactory.getLicense(document);

            needGPL = !license.wasAccepted();

            file = XMLSerializer.getFilePath(NaN.LICENSE_DIRECTORY,
                                             NaN.MINDTERM_END_USER_FILE);
            document = XMLSerializer.readDocument(file);
            license = XMLFactory.getLicense(document);

            needEndUser = !license.wasAccepted();

            file = XMLSerializer.getFilePath(NaN.LICENSE_DIRECTORY,
                                             NaN.MINDTERM_SOURCE_FILE);
            document = XMLSerializer.readDocument(file);
            license = XMLFactory.getLicense(document);

            needSource = !license.wasAccepted();
        } catch (XMLParseException e) {
            // probably just a 404 not found error
        }

        LicenseDialog dialog = new LicenseDialog(this, true);

        if (needGPL) {
            boolean accepted = displayLicense(dialog, NaN.FREE_LICENSE,
                                              NaN.FREE_FILE);

            if (!accepted) {
                exit(false);
            }
        }

        if (needEndUser) {
            boolean accepted = displayLicense(dialog,
                                              NaN.MINDTERM_END_USER_LICENSE,
                                              NaN.MINDTERM_END_USER_FILE);

            if (!accepted) {
                exit(false);
            }
        }

        if (needSource) {
            boolean accepted = displayLicense(dialog,
                                              NaN.MINDTERM_SOURCE_LICENSE,
                                              NaN.MINDTERM_SOURCE_FILE);

            if (!accepted) {
                exit(false);
            }
        }

        dialog.dispose();
    }

    /**
     * Connects or reconnects to the NNTP server.
     *
     * @param close If the current connection should be closed first.
     */
    private void reconnect(boolean close) {
        if (close) {
            SSHConnection.getInstance().close();
            this.client.disconnect();
        }

        WaitDialog dialog = new WaitDialog(this);
        BackgroundConnectThread thread = new BackgroundConnectThread(settings.getServerSettings(),
                                                                     client,
                                                                     dialog);
        thread.start();
        addBackgroundThread(thread);

        while (!thread.isFinished()) {
            dialog.display("Opening NNTP connection", "Please wait...");
        }

        dialog.dispose();

        if (!this.client.isConnected()) {
            JOptionPane.showMessageDialog(this,
                                          "Unable to connect to NNTP server.\n" +
                                          "Check the server settings and try again...",
                                          "NNTP Connection Error",
                                          JOptionPane.ERROR_MESSAGE);
        }
    }

    /**
     * Shows the User Settings Dialog.
     */
    public void showUserSettingsDialog() {
        File file = XMLSerializer.getFilePath(NaN.SETTINGS_DIRECTORY,
                                              NaN.USER_SETTINGS_FILE);

        this.userSettingsDialog.display();

        if (!file.exists() || this.userSettingsDialog.isModified()) {
            // background write the settings
            BackgroundWriter writer = new BackgroundWriter(settings.getUserSettings(),
                                                           file);
            writer.start();
            addBackgroundThread(writer);
        }
    }

    /**
     * Shows the Server Settings dialog.
     */
    public void showServerSettingsDialog() {
        boolean checkForDownload = !settings.getServerSettings()
                                            .useRemoteSave();

        File file = XMLSerializer.getFilePath(NaN.SETTINGS_DIRECTORY,
                                              NaN.SERVER_SETTINGS_FILE);

        this.serverSettingsDialog.display();

        if (settings.getServerSettings().useSSH()) {
            SSHConnection ssh = SSHConnection.getInstance();

            if (ssh.isConnected()) {
                String hostname = settings.getServerSettings().getSSHServer();
                int port = settings.getServerSettings().getSSHPort();

                if (!ssh.isConnectedTo(hostname, port)) {
                    ssh.close();
                    ssh.connect(settings.getServerSettings());
                }
            }

            if (checkForDownload &&
                settings.getServerSettings().useRemoteSave()) {
                // ask them if we should download their settings from server
                int choice;

                choice = JOptionPane.showConfirmDialog(this,
                                                       "Download current settings from server?",
                                                       "Remote Load?",
                                                       JOptionPane.YES_NO_OPTION);

                if (choice == JOptionPane.YES_OPTION) {
                    if (ssh.isConnected()) {
                        ssh.download(settings.getServerSettings()
                                             .getDirectory(),
                                     NaN.ARCHIVE_FILE.getName());

                        if (NaN.ARCHIVE_FILE.exists()) {
                            SettingsArchiver.unarchive();
                            NaN.ARCHIVE_FILE.delete();
                            settings.reloadSettings();
                        } else {
                            // unable to load file
                            // don't save it later...
                            settings.getServerSettings().setDirectory(null);
                        }
                    }
                }
            }
        }

        if (!file.exists() || this.serverSettingsDialog.isModified()) {
            // connect to the NNTP server
            this.reconnect(true);

            // background write the settings
            BackgroundWriter writer = new BackgroundWriter(settings.getServerSettings(),
                                                           file);
            writer.start();
            addBackgroundThread(writer);
        }
    }

    /**
     * Shows the Newsgroup Dialog.
     */
    public void showNewsgroupDialog() {
        if (!settings.isAvailableNewsgroupsReady()) {
            WaitDialog dialog = new WaitDialog(this);
            settings.setWaitDialog(dialog);

            while (!settings.isAvailableNewsgroupsReady()) {
                dialog.display("Reading Settings",
                               "Please wait while we finish reading your settings...");
            }

            dialog.dispose();
        }

        this.newsgroupDialog.display(settings.getAvailableNewsgroups(),
                                     settings.getSubscribedNewsgroups());

        if (this.newsgroupDialog.wasAvailableModified()) {
            // serialize available
            NewsgroupList list = settings.getAvailableNewsgroups();
            File file = XMLSerializer.getFilePath(NaN.SETTINGS_DIRECTORY,
                                                  NaN.AVAILABLE_NEWSGROUPS_FILE);

            BackgroundWriter writer = new BackgroundWriter(list, file);
            writer.start();

            addBackgroundThread(writer);
        }

        if (this.newsgroupDialog.wasSubscribedModified()) {
            // serialize subscribed
            NewsgroupList list = settings.getSubscribedNewsgroups();
            list.sort();

            File file = XMLSerializer.getFilePath(NaN.SETTINGS_DIRECTORY,
                                                  NaN.SUBSCRIBED_NEWSGROUPS_FILE);

            BackgroundWriter writer = new BackgroundWriter(list, file);
            writer.start();

            addBackgroundThread(writer);
        }
    }

    /**
     * Displays the help frame.
     */
    public void showHelp() {
        this.help.setVisible(true);
    }

    /**
     * Called when the user requests to change the Newsgroup Messages to
     * cache locally.
     */
    public void changeNewsgroupMessages() {
        if (currentNewsgroup != null) {
            int messages = -1;

            do {
                String text = JOptionPane.showInputDialog(this,
                                                          "Enter the number of messages to cache" +
                                                          " locally.\nUse 0 for all messages.");

                try {
                    messages = Integer.parseInt(text);
                } catch (NumberFormatException e) {
                }

                if (messages < 0) {
                    JOptionPane.showMessageDialog(this,
                                                  "Messages must be a postive integer.",
                                                  "Invalid Message Number",
                                                  JOptionPane.ERROR_MESSAGE);
                }
            } while (messages < 0);

            currentNewsgroup.setMessages(messages);

            // serialize the new newsgroup file
            File file = BackgroundCacheWriter.getGroupFile(currentNewsgroup);
            XMLSerializer.write(currentNewsgroup.toXML(), file);

            // older messages may be the parents of current ones
            // we can't trust the old cache...
            currentNewsgroup.setCache(null);

            // delete the old cache...
            File directory = BackgroundCacheWriter.getCacheDirectory(currentNewsgroup);

            BackgroundDirectoryDeleter thread = new BackgroundDirectoryDeleter(directory);
            thread.start();

            // cannot continue until this task is done...
            // shouldn't take long though...
            // unless maybe there are 5000+ messages on a network server or something...
            while (!thread.isFinished()) {
                try {
                    Thread.sleep(50);
                } catch (InterruptedException e) {
                    // this is not a problem...
                }
            }

            MessageTree cache = new MessageTree();
            WaitDialog dialog = new WaitDialog(this);

            this.loadMessagesFromServer(dialog, cache);

            dialog.dispose();

            currentNewsgroup.setCache(cache);
            panel.setMessageTree(cache);
        }
    }

    /**
     * Downloads message bodies and caches the current newsgroup.
     */
    public void cacheCurrentNewsgroup() {
        if ((currentNewsgroup != null) && (this.client.isConnected())) {
            WaitDialog dialog = new WaitDialog(this);
            BackgroundCacheDownloader thread = new BackgroundCacheDownloader(this,
                                                                             currentNewsgroup,
                                                                             client,
                                                                             dialog);
            thread.start();

            while (!thread.isFinished()) {
                dialog.display("Downloading Newsgroup", "Please wait...");
            }

            dialog.dispose();

            panel.setMessageTree(currentNewsgroup.getCache());
        }
    }

    /**
     * Refreshes the current Newsgroup cache.
     */
    public void refreshCurrentNewsgroup() {
        if (this.client.isConnected() && (this.currentNewsgroup != null)) {
            updateMessageCache(currentNewsgroup);
            panel.setMessageTree(currentNewsgroup.getCache());
        }
    }

    /**
     * Refreshes the available Newsgroup list.
     */
    public void refreshNewsgroupList() {
        if (this.client.isConnected() &&
            (this.settings.getAvailableNewsgroups().getSize() > 0)) {
            updateNewsgroupList();
        }
    }

    /**
     * Shows the current message's source.
     */
    public void showCurrentMessageSource() {
        if (this.currentMessage != null) {
            SourceDialog dialog = new SourceDialog(this);

            MessageHeader header = currentMessage.getHeader();

            String title = "Source for message " +
                           header.getValue("message-id");
            String source = header.getHeaders() + '\n' +
                            currentMessage.getBody();

            dialog.display(title, source);
        }
    }

    /**
     * Posts a new message to the current newsgroup.
     */
    public void postMessage() {
        if (!this.client.isConnected()) {
            JOptionPane.showMessageDialog(this,
                                          "You must be connected into order to post messages.",
                                          "Error: Not connected to server",
                                          JOptionPane.ERROR_MESSAGE);
        } else if (this.currentNewsgroup == null) {
            JOptionPane.showMessageDialog(this,
                                          "You must select a Newsgroup before you can post.",
                                          "Error: No current Newsgroup",
                                          JOptionPane.ERROR_MESSAGE);
        } else {
            ComposeFrame frame = new ComposeFrame(this);

            String from = settings.getUserSettings().getName();
            String email = settings.getUserSettings().getEmail();
            String signature = settings.getUserSettings().getSignature();

            if (email.length() > 0) {
                from += (' ' + ('<' + email + '>'));
            }

            frame.display(from, currentNewsgroup.getName(), signature);
        }
    }

    /**
     * Replies to the current message.
     */
    public void replyToMessage() {
        if (!this.client.isConnected()) {
            JOptionPane.showMessageDialog(this,
                                          "You must be connected into order to post messages.",
                                          "Error: Not connected to server",
                                          JOptionPane.ERROR_MESSAGE);
        } else if (this.currentMessage == null) {
            JOptionPane.showMessageDialog(this,
                                          "You must select a Message before you can reply.",
                                          "Error: No current Newsgroup",
                                          JOptionPane.ERROR_MESSAGE);
        } else {
            ComposeFrame frame = new ComposeFrame(this);

            String newsgroup = currentNewsgroup.getName();

            String from = settings.getUserSettings().getName();
            String email = settings.getUserSettings().getEmail();
            String signature = settings.getUserSettings().getSignature();

            if (email.length() > 0) {
                from += (' ' + ('<' + email + '>'));
            }

            String references = "";
            MessageHeader header = currentMessage.getHeader();

            if (header.containsHeader("references")) {
                references = header.getValue("references") + ' ';
            }

            references += header.getValue("message-id");

            String subject = header.getValue("subject");

            if (!subject.toLowerCase().startsWith("re:")) {
                subject = "Re: " + subject;
            }

            frame.display(from, newsgroup, subject, signature,
                          currentMessage.getQuotedBody(), references);
        }
    }

    /**
     * Updates the list of available Newsgroups.
     */
    private void updateNewsgroupList() {
        NewsgroupList available = settings.getAvailableNewsgroups();
        Date when = available.getUpdated();
        available.setUpdated();

        NewsgroupList recent = this.client.getNewsgroupsSince(when);
        available.append(recent);
        available.sort();

        File file = XMLSerializer.getFilePath(NaN.SETTINGS_DIRECTORY,
                                              NaN.AVAILABLE_NEWSGROUPS_FILE);
        BackgroundWriter writer = new BackgroundWriter(available, file);
        writer.start();

        addBackgroundThread(writer);
    }

    /**
     * Updates the message cache with more recent additions to the group.
     *
     * @param newsgroup The Newsgroup to update.
     */
    private void updateMessageCache(Newsgroup newsgroup) {
        // search for new Messages
        this.client.selectNewsgroup(newsgroup);

        for (int i = newsgroup.getNextArticle();
             i <= newsgroup.getLastArticle(); i++) {
            try {
                MessageHeader header = client.getHeader(i);
                MessageTree tree = newsgroup.getCache();

                if (header.containsHeader("references")) {
                    String references = header.getValue("references");
                    StringTokenizer tokens = new StringTokenizer(references);

                    tree.addThreadChild(header, tokens);
                } else {
                    tree.addChild(header);
                }

                File file = BackgroundCacheWriter.getCacheFile(newsgroup,
                                                               header);
                BackgroundWriter writer = new BackgroundWriter(header, file);
                writer.start();
                addBackgroundThread(writer);
            } catch (InvalidArticleException e) {
                // this happens occasionally...
                EventLogger.getInstance().logError("InvalidArticleException in " +
                                                   "View.updateMessageCache()...");
            }
        }

        // we should now be current...
        newsgroup.setNextArticle(newsgroup.getLastArticle() + 1);

        // serialize new group file
        File file = BackgroundCacheWriter.getGroupFile(newsgroup);
        BackgroundWriter writer = new BackgroundWriter(newsgroup, file);
        writer.start();

        addBackgroundThread(writer);
    }

    /**
     * Loads the Newsgroup messages from the server.
     *
     * @param dialog The WaitDialog to display.
     * @param tree The MessageTree to place the messages in.
     */
    private void loadMessagesFromServer(WaitDialog dialog, MessageTree tree) {
        BackgroundNewsgroupDownloader thread = new BackgroundNewsgroupDownloader(client,
                                                                                 dialog,
                                                                                 currentNewsgroup,
                                                                                 tree);
        thread.start();
        addBackgroundThread(thread);

        currentNewsgroup.setUpdated();

        while (!thread.isFinished()) {
            dialog.display("Downloading message headers...", "Please wait...");
        }

        // background serialize the cache
        BackgroundCacheWriter writer = new BackgroundCacheWriter(this.currentNewsgroup,
                                                                 tree);
        writer.start();
        addBackgroundThread(writer);

        this.currentNewsgroup.setCache(tree);
    }

    /**
     * Displays the selected Newsgroup. DO NOT CALL DIRECTLY. THIS SHOULD
     * ONLY BE CALLED BY valueChanged(ListSelectionListener).
     */
    private void displayGroup() {
        WaitDialog dialog = new WaitDialog(this);
        MessageTree tree = new MessageTree();

        File directory = BackgroundCacheWriter.getCacheDirectory(currentNewsgroup);

        if (this.currentNewsgroup.hasCache()) {
            tree = this.currentNewsgroup.getCache();

            this.currentNewsgroup.setCache(tree);

            if (this.client.isConnected()) {
                updateMessageCache(currentNewsgroup);
            }
        } else if (directory.exists()) {
            // read from cache
            BackgroundCacheReader reader = new BackgroundCacheReader(directory,
                                                                     currentNewsgroup,
                                                                     tree,
                                                                     dialog);
            reader.start();
            addBackgroundThread(reader);

            while (!reader.isFinished()) {
                dialog.display("Reading messages from cache",
                               "Please wait...");
            }

            this.currentNewsgroup.setCache(tree);

            if (this.client.isConnected()) {
                updateMessageCache(currentNewsgroup);
            }
        } else {
            // load from Server
            if (this.client.isConnected()) {
                this.loadMessagesFromServer(dialog, tree);
            }
        }

        dialog.dispose();

        // set the updated Tree...
        if (currentNewsgroup.getCache() != null) {
            this.panel.setMessageTree(currentNewsgroup.getCache());
        } else {
            this.panel.clear();
        }
    }

    /**
     * Called when a user selects a Newsgroup from the subscribed group
     * list.
     *
     * @param event The event that triggered this callback.
     */
    public void valueChanged(ListSelectionEvent event) {
        if (!event.getValueIsAdjusting()) {
            JList list = (JList)event.getSource();
            Newsgroup newsgroup = (Newsgroup)list.getSelectedValue();

            if ((newsgroup != null) && (newsgroup != this.currentNewsgroup)) {
                if (this.client.isConnected()) {
                    this.client.selectNewsgroup(newsgroup);
                }

                this.currentNewsgroup = newsgroup;
                displayGroup();
            } else if (newsgroup == null) {
                // removed current selection...
                this.panel.clear();
            }
        }
    }

    /**
     * Called when a node on the MessageTree is selected.
     *
     * @param event The associated TreeSelectionEvent.
     */
    public void valueChanged(TreeSelectionEvent event) {
        JTree tree = (JTree)event.getSource();
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)tree.getLastSelectedPathComponent();

        if (node != null) {
            Object object = node.getUserObject();

            Message message = null;

            if (object instanceof MessageHeader) {
                MessageHeader header = (MessageHeader)object;
                String messageId = header.getValue("message-id");

                try {
                    if (this.client.isConnected()) {
                        message = this.client.getMessage(messageId, header);
                        message.setRead(true);
                        node.setUserObject(message);

                        File file = BackgroundCacheWriter.getCacheFile(currentNewsgroup,
                                                                       message.getHeader());
                        BackgroundWriter writer = new BackgroundWriter(message,
                                                                       file);
                        writer.start();
                        addBackgroundThread(writer);
                    }
                } catch (InvalidArticleException e) {
                    // this shouldn't happen often, but it could happen...
                    EventLogger.getInstance().logMessage("Possible message deletion... Caught " +
                                                         "InvalidArticleException in View.valueChanged(TreeSelectionEvent)...");
                }
            } else if (object instanceof Message) {
                message = (Message)object;

                if (!message.isRead()) {
                    message.setRead(true);

                    File file = BackgroundCacheWriter.getCacheFile(currentNewsgroup,
                                                                   message.getHeader());
                    BackgroundWriter writer = new BackgroundWriter(message,
                                                                   file);
                    writer.start();
                    addBackgroundThread(writer);
                }
            } else {
                EventLogger.getInstance().logError("Selected tree node contains neither " +
                                                   "a Message nor a MessageHeader!");
            }

            this.currentMessage = message;

            if (message != null) {
                this.panel.display(message);
            }
        }
    }

    /**
     * Called when the mouse is pressed (clicked &amp; released).
     *
     * @param event The event that triggered this callback.
     */
    public void mousePressed(MouseEvent event) {
        Object source = event.getSource();

        if (event.isPopupTrigger()) {
            if (source instanceof JList) {
                JList list = (JList)source;
                newsgroupPopup.show(list, event.getX(), event.getY());
            } else if (source instanceof JTreeTable) {
                JTreeTable tree = (JTreeTable)source;
                messagePopup.show(tree, event.getX(), event.getY());
            } else {
                EventLogger.getInstance().logError("Mouse click event source is neither " +
                                                   "JList nor JTree!");
            }
        }
    }

    /**
     * Called when the mouse is released.
     *
     * @param event The event that triggered this callback.
     */
    public void mouseReleased(MouseEvent event) {
        Object source = event.getSource();

        if (event.isPopupTrigger()) {
            if (source instanceof JList) {
                JList list = (JList)source;
                newsgroupPopup.show(list, event.getX(), event.getY());
            } else if (source instanceof JTreeTable) {
                JTreeTable tree = (JTreeTable)source;
                messagePopup.show(tree, event.getX(), event.getY());
            } else {
                EventLogger.getInstance().logError("Mouse click event source is neither " +
                                                   "JList nor JTree!");
            }
        }
    }

    /**
     * Called when the window is being closed.
     *
     * @param event The associated WindowEvent.
     */
    public void windowClosing(WindowEvent event) {
        exit(true);
    }

    /**
     * Called when the window is opened.
     *
     * @param event The associated WindowEvent.
     */
    public void windowOpened(WindowEvent event) {
        // display the dialogs if the settings were not loaded
        if (!this.settings.loadedUserSettings()) {
            this.showUserSettingsDialog();
        }

        if (!this.settings.loadedServerSettings()) {
            this.showServerSettingsDialog();
        } else if (this.settings.getServerSettings().useSSH() &&
                   (this.settings.getServerSettings().getPassword().length() == 0)) {
            PasswordDialog dialog = new PasswordDialog(this,
                                                       settings.getServerSettings());
            dialog.display();

            this.reconnect(false);
        } else {
            this.reconnect(false);
        }

        // check for newsgroup list updates...
        this.refreshNewsgroupList();
    }

    /**
     * Called when the window is closed (finished closing).
     *
     * @param event The associated WindowEvent.
     */
    public void windowClosed(WindowEvent event) {
    }

    /**
     * Called when the window is activated.
     *
     * @param event The associated WindowEvent.
     */
    public void windowActivated(WindowEvent event) {
    }

    /**
     * Called when the window is iconified (minimized).
     *
     * @param event The associated WindowEvent.
     */
    public void windowIconified(WindowEvent event) {
    }

    /**
     * Called when the window is deactivated (lost focus, deselected).
     *
     * @param event The associated WindowEvent.
     */
    public void windowDeactivated(WindowEvent event) {
    }

    /**
     * Called when the window is deiconified (un-minimized, restored).
     *
     * @param event The associated WindowEvent.
     */
    public void windowDeiconified(WindowEvent event) {
    }

    /**
     * Called when the mouse is clicked.
     *
     * @param event The event that triggered this callback.
     */
    public void mouseClicked(MouseEvent event) {
    }

    /**
     * Called when the mouse enters an area.
     *
     * @param event The event that triggered this callback.
     */
    public void mouseEntered(MouseEvent event) {
    }

    /**
     * Called when the mouse exits an area.
     *
     * @param event The event that triggered this callback.
     */
    public void mouseExited(MouseEvent event) {
    }

    /**
     * Returns a String representation of this object.
     *
     * @return A String representation.
     */
    public String toString() {
        return "View";
    }
}
